include_guard(GLOBAL)

function(edx_check_target_arch)
if("${CMAKE_C_COMPILER_ID}" MATCHES "GNU|Clang")
	execute_process(
		COMMAND ${CMAKE_C_COMPILER} -v
		OUTPUT_VARIABLE  C_COMPILER_V_OUTPUT
		ERROR_VARIABLE  C_COMPILER_V_OUTPUT
	)
	string(REGEX MATCH "Target:[ \\t]*([^ \t\n]*)" GCC_TARGET "${C_COMPILER_V_OUTPUT}")
	set(GCC_TARGET ${CMAKE_MATCH_1})
	string(REGEX MATCH "([^-]*)-([^-]*)-(.*)" GCC_TARGET_ARCH "${GCC_TARGET}")
	set(GCC_TARGET_ARCH ${CMAKE_MATCH_1})
	string(REPLACE "x86_64" "x64" GCC_TARGET_ARCH ${GCC_TARGET_ARCH})
	set(GCC_TARGET_OS ${CMAKE_MATCH_2})
	set(GCC_TARGET_ABI ${CMAKE_MATCH_3})

	if ("${CMAKE_C_COMPILER_PREDEFINES_COMMAND}" MATCHES ".*(gcc|clang).*-m32.*")
		set(GCC_TARGET_ARCH i686)
	elseif ("${CMAKE_CXX_COMPILER_PREDEFINES_COMMAND}" MATCHES ".*(g|clang)\\+\\+.*-m32.*")
		set(GCC_TARGET_ARCH i686)
	endif()

	string(REGEX MATCH "gcc version[ \\t]*([0-9\\.]+)" GCC_TARGET "${C_COMPILER_V_OUTPUT}")
	set(GCC_VERSION ${CMAKE_MATCH_1})

	if ("${CMAKE_CXX_COMPILER_ARCHITECTURE_ID}" STREQUAL "")
		set(TARGET_ARCH ${GCC_TARGET_ARCH})
	else ()
		set(TARGET_ARCH ${CMAKE_CXX_COMPILER_ARCHITECTURE_ID})
	endif ()


	if ("${TARGET_ARCH}" MATCHES "[Ll]inux")
		set(TARGET_ABI "gnu")
	elseif("${TARGET_ARCH}" STREQUAL "i686")
		set(TARGET_ARCH x86)
		set(TARGET_ABI ${GCC_TARGET_ABI})
	elseif("${GCC_TARGET_ABI}" MATCHES "[Ll]inux")
		set(TARGET_ABI "gnu")
	else()
		set(TARGET_ABI ${GCC_TARGET_ABI})
	endif ()

	unset(C_COMPILER_V_OUTPUT)
elseif("${CMAKE_C_COMPILER_ID}" STREQUAL "IntelLLVM")
	set(TARGET_ARCH ${CMAKE_C_COMPILER_ARCHITECTURE_ID})
elseif("${CMAKE_C_COMPILER_ID}" STREQUAL "MSVC")
	set(TARGET_ARCH ${CMAKE_C_COMPILER_ARCHITECTURE_ID})
endif()

if ("${TARGET_ARCH}" MATCHES  "[xX]64")
	if (${CMAKE_C_COMPILER_ID} STREQUAL "Clang")
		set(_EDX_MSYS_CURRENT_HOST clang64 PARENT_SCOPE)
	else()
		set(_EDX_MSYS_CURRENT_HOST mingw64 PARENT_SCOPE)
	endif()
else()
	if (${CMAKE_C_COMPILER_ID} STREQUAL "Clang")
		set(_EDX_MSYS_CURRENT_HOST clang32 PARENT_SCOPE)
	else()
		set(_EDX_MSYS_CURRENT_HOST mingw32 PARENT_SCOPE)
	endif()
endif()
endfunction()

function(edx_install_pkg)
set(_EDX_ESCAPE "[")
set(_ESC_ERR "${_EDX_ESCAPE}31m")
set(_ESC_PKG "${_EDX_ESCAPE}34m")
set(_ESC_URL "${_EDX_ESCAPE}35m")
set(_ESC_RST "${_EDX_ESCAPE}0m")
set(_CURRENT_PKG_NAME "${ARGV0}")
set(_CURRENT_PKG_FILE "${ARGV1}")
set(_CURRENT_PKG_VER "${ARGV2}")
set(_CURRENT_PKG_SIZE "${ARGV3}")
set(_CURRENT_PKG_MD5 "${ARGV4}")
set(_EDX_PKG_INSTALL_INFO_ROOT "${_EDX_PKG_ROOT}/${_EDX_MSYS_CURRENT_HOST}/installed")
set(_EDX_PKG_INSTALL_INFO "${_EDX_PKG_INSTALL_INFO_ROOT}/${_CURRENT_PKG_NAME}.txt")
unset(_EDX_SHOULD_EXTRACT_PKG)
set(_EDX_CRT_LIST "crt-git;crt")
set(_EDX_HEADERS_LIST "headers-git;headers")

if ("${_CURRENT_PKG_NAME}" IN_LIST _EDX_CRT_LIST)
	if (${CMAKE_C_COMPILER_ID} STREQUAL "Clang")
		set(_EDX_EXTRACT_PATTERN "clang*/*.a")
	else()
		set(_EDX_EXTRACT_PATTERN "mingw*/*.a")
	endif()
elseif ("${_CURRENT_PKG_NAME}" IN_LIST _EDX_HEADERS_LIST)
	return() # do not install mingw headers
else()
	if (${CMAKE_C_COMPILER_ID} STREQUAL "Clang")
		set(_EDX_EXTRACT_PATTERN "clang*/*")
	else()
		set(_EDX_EXTRACT_PATTERN "mingw*/*")
	endif()
endif()

if ("${_CURRENT_PKG_SIZE}" STREQUAL "")
	# file size is empty
elseif (EXISTS "${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}")
	file(SIZE "${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}" _CURRENT_PKG_REAL_SIZE)
	if (NOT "${_CURRENT_PKG_REAL_SIZE}" STREQUAL "${_CURRENT_PKG_SIZE}")
		# file size not matched
		file(REMOVE "${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}")
		message("Fix the corrupted package: ${_ESC_ERR}${_CURRENT_PKG_NAME}${_ESC_RST}")
	endif()
endif()

if (NOT EXISTS "${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}")
	message("Download ${_ESC_PKG}${_CURRENT_PKG_NAME}${_ESC_RST} : ${_ESC_URL}${_EDX_MSYS_HOST_URL}/${_EDX_MSYS_CURRENT_HOST}/${_CURRENT_PKG_FILE}${_ESC_RST}")
	file(DOWNLOAD
		"${_EDX_MSYS_HOST_URL}/${_EDX_MSYS_CURRENT_HOST}/${_CURRENT_PKG_FILE}"
		"${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}"
		STATUS _EDX_DOWNLOAD_STATUS
	)

	if (NOT "0" IN_LIST _EDX_DOWNLOAD_STATUS)
		message(FATAL_ERROR "${_ESC_ERR}Error${_ESC_RST}: failed to download ${_ESC_PKG}${_CURRENT_PKG_NAME}${_ESC_RST} from ${_ESC_URL}${_EDX_MSYS_HOST_URL}/${_EDX_MSYS_CURRENT_HOST}/${_CURRENT_PKG_FILE}${_ESC_RST}")
	endif()

	if (EXISTS "${_EDX_PKG_INSTALL_INFO}")
		set(_EDX_SHOULD_EXTRACT_PKG "Updating")
	else()
		set(_EDX_SHOULD_EXTRACT_PKG "Extracting")
	endif()
elseif(EXISTS "${_EDX_PKG_INSTALL_INFO}")
	file(READ "${_EDX_PKG_INSTALL_INFO}" _EDX_OLD_PKG)
	if (NOT "${_EDX_OLD_PKG}" STREQUAL "${_CURRENT_PKG_FILE}")
		set(_EDX_SHOULD_EXTRACT_PKG "Updating")
	endif()
else()
	set(_EDX_SHOULD_EXTRACT_PKG "Extracting")
endif()

if (_EDX_SHOULD_EXTRACT_PKG)
	message("${_EDX_SHOULD_EXTRACT_PKG} ${_ESC_PKG}${_CURRENT_PKG_NAME}${_ESC_RST}...")
	file(ARCHIVE_EXTRACT
		INPUT "${_EDX_PKG_CACHE}/${_CURRENT_PKG_FILE}"
		PATTERNS ${_EDX_EXTRACT_PATTERN}
		DESTINATION "${_EDX_PKG_ROOT}"
	)
	file(MAKE_DIRECTORY "${_EDX_PKG_ROOT}/${_EDX_MSYS_CURRENT_HOST}/installed")
	file(WRITE
		"${_EDX_PKG_INSTALL_INFO}"
		"${_CURRENT_PKG_FILE}"
	)
endif()
endfunction()

function(edx_map_libs)
set(_EDX_OUT_LIST )
foreach(_NAME IN LISTS ${ARGV0})
list(FIND ${ARGV1} "${_NAME}" _IDX)
if (_IDX GREATER "-1")
	math(EXPR _PATH_IDX "${_IDX} + 1")
	list(GET ${ARGV1} ${_PATH_IDX} _ITEM_PATH)
	list(APPEND _EDX_OUT_LIST "${_ITEM_PATH}")
else()
	list(APPEND _EDX_OUT_LIST "${_NAME}")
endif()
endforeach()
set(${ARGV0} "${_EDX_OUT_LIST}" PARENT_SCOPE)
endfunction()

function(edx_map_static_libs)
set(_EDX_OUT_LIST )
set(_EDX_GCC_LIBS "stdc++;pthread;atomic;gomp;")
set(_EDX_SYSTEM_LIBS "user32;gdi32;kernel32;winmm;advapi32;bcrypt;usp10;rpcrt4;msimg32;d2d1;windowscodecs;dwrite;ole32;iphlpapi;m;ws2_32;crypt32;dnsapi;uuid;shlwapi;userenv;ncrypt;version;pathcch;gcc_s;gcc;shell32;wsock32;dinput8;setupapi;mingw32;moldname;mingwex;gdiplus;imm32;oleaut32;wldap32;")
foreach(_NAME IN LISTS ${ARGV0})
if ("${_NAME}" MATCHES ".*\\.dll\\.a$")
	string(REPLACE ".dll.a" ".a" _NAME_STATIC ${_NAME})
	if ("${_NAME_STATIC}" IN_LIST ${ARGV1})
		list(APPEND _EDX_OUT_LIST "${_NAME_STATIC}")
	else()
		list(APPEND _EDX_OUT_LIST "${_NAME}")
	endif()
elseif ("${_NAME}" MATCHES ".*\\.a$")
	list(APPEND _EDX_OUT_LIST "${_NAME}")
elseif("${_NAME}" IN_LIST _EDX_GCC_LIBS)
	list(APPEND _EDX_OUT_LIST "lib${_NAME}.a")
elseif("${_NAME}" IN_LIST _EDX_SYSTEM_LIBS)
	list(APPEND _EDX_OUT_LIST "${_NAME}")
else()
	list(APPEND _EDX_OUT_LIST "${_NAME}-static")
endif()
endforeach()
set(${ARGV0} "${_EDX_OUT_LIST}" PARENT_SCOPE)
endfunction()

function(edx_read_pkg_config)
set(_EDX_PC_FILE "${ARGV0}")
set(${ARGV1} PARENT_SCOPE)	# _EDX_PUB_LIBS
set(${ARGV2} PARENT_SCOPE)	# _EDX_PRIVATE_LIBS
set(${ARGV3} PARENT_SCOPE)	# _EDX_INC_PATHS
set(${ARGV4} PARENT_SCOPE)	# _EDX_DEFS
set(${ARGV5} PARENT_SCOPE)	# _EDX_CFLAGS
set(${ARGV6} PARENT_SCOPE)	# _EDX_LDFLAGS
set(${ARGV7} PARENT_SCOPE)	# _EDX_DEPS

if ("${_EDX_PC_FILE}" MATCHES ".*(haisrt|srt).pc")
	set(_EDX_LIB_FILTER "mingw32|shaderc_util|dl|fcgi|curl")
else()
	set(_EDX_LIB_FILTER "shaderc_util|dl|fcgi|curl")
endif()

if (EXISTS "${_EDX_PC_FILE}")
	file(STRINGS "${_EDX_PC_FILE}" _EDX_PKG_CONFIG_DATA)
	foreach(_EDX_CURRENT_LINE IN LISTS _EDX_PKG_CONFIG_DATA)
	if ("${_EDX_CURRENT_LINE}" MATCHES "^prefix=(.+)$")
		set(_EDX_PREFIX ${CMAKE_MATCH_1})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^target=(.+)$")
		set(_EDX_TARGET ${CMAKE_MATCH_1})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^libdir=(.+)$")
		set(_EDX_LIB_DIR ${CMAKE_MATCH_1})
		string(REGEX REPLACE "\\\${prefix}" "${_EDX_PREFIX}" _EDX_LIB_DIR ${_EDX_LIB_DIR})
		string(REGEX REPLACE "\\\${exec_prefix}" "${_EDX_EXE_PREFIX}" _EDX_LIB_DIR ${_EDX_LIB_DIR})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^exec_prefix=(.+)$")
		set(_EDX_EXE_PREFIX ${CMAKE_MATCH_1})
		string(REGEX REPLACE "\\\${prefix}" "${_EDX_PREFIX}" _EDX_EXE_PREFIX ${_EDX_EXE_PREFIX})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^includedir=(.+)$")
		set(_EDX_INCLUDE_DIR ${CMAKE_MATCH_1})
		string(REGEX REPLACE "\\\${prefix}" "${_EDX_PREFIX}" _EDX_INCLUDE_DIR ${_EDX_INCLUDE_DIR})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^APRUTIL_MAJOR_VERSION=(.+)$")
		set(_EDX_APRUTIL_MAJOR_VERSION ${CMAKE_MATCH_1})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^APR_MAJOR_VERSION=(.+)$")
		set(_EDX_APR_MAJOR_VERSION ${CMAKE_MATCH_1})
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^Cflags:(.+)$")
		set(_EDX_CURRENT_MATCH ${CMAKE_MATCH_1})
		string(REGEX MATCHALL "-I([^ ]+)" _EDX_INC_PATHS ${_EDX_CURRENT_MATCH})
		string(REGEX MATCHALL "-D([^ ]+)" _EDX_DEFS ${_EDX_CURRENT_MATCH})

		list(TRANSFORM _EDX_INC_PATHS REPLACE "-I([^ ]+)" "\\1")
		list(TRANSFORM _EDX_DEFS REPLACE "-D([^ ]+)" "\\1")
		list(TRANSFORM _EDX_INC_PATHS REPLACE "\\\${includedir}" "${_EDX_INCLUDE_DIR}")
		list(TRANSFORM _EDX_INC_PATHS REPLACE "\\\${libdir}" "${_EDX_LIB_DIR}")
		if (${_EDX_APR_MAJOR_VERSION})
			list(TRANSFORM _EDX_INC_PATHS REPLACE "\\\${APR_MAJOR_VERSION}" "${_EDX_APR_MAJOR_VERSION}")
		endif()
		if (${_EDX_APRUTIL_MAJOR_VERSION})
			list(TRANSFORM _EDX_INC_PATHS REPLACE "\\\${APRUTIL_MAJOR_VERSION}" "${_EDX_APRUTIL_MAJOR_VERSION}")
		endif()

		list(TRANSFORM _EDX_INC_PATHS PREPEND "${_EDX_PKG_ROOT}")
		list(APPEND _EDX_INC_PATHS ${_EDX_PKG_ROOT}/${_EDX_MSYS_CURRENT_HOST}/include)
		list(REMOVE_DUPLICATES _EDX_INC_PATHS)

		set(${ARGV3} ${_EDX_INC_PATHS} PARENT_SCOPE)
		set(${ARGV4} ${_EDX_DEFS} PARENT_SCOPE)
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^Requires:[ \t]*(.+)$")
		set(_EDX_CURRENT_MATCH ${CMAKE_MATCH_1})
		if (NOT "${_EDX_CURRENT_MATCH}" MATCHES "#.*")
			string(REGEX MATCHALL "([^ \t,]+)[ \t]*[><=]*[ \t]*[0-9.]*,?" _EDX_PUB_DEPS ${_EDX_CURRENT_MATCH})

			list(TRANSFORM _EDX_PUB_DEPS REPLACE "([^ \t,]+)[ \t]*[><=]*[ \t]*[0-9.]*,?" "\\1")
			list(TRANSFORM _EDX_PUB_DEPS REPLACE "\\\${includedir}" "${_EDX_INCLUDE_DIR}")
			list(TRANSFORM _EDX_PUB_DEPS REPLACE "\\\${libdir}" "${_EDX_LIB_DIR}")
			list(TRANSFORM _EDX_PUB_DEPS REPLACE "\\\${target}" "${_EDX_TARGET}")
			list(APPEND _EDX_DEPS ${_EDX_PUB_DEPS})
		endif()
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^Requires.private:[ \t]*(.+)$")
		set(_EDX_CURRENT_MATCH ${CMAKE_MATCH_1})
		if (NOT "${_EDX_CURRENT_MATCH}" MATCHES "#.*")
			string(REGEX MATCHALL "([^ \t,]+)[ \t]*[><=]*[ \t]*[0-9.]*,?" _EDX_PRIVATE_DEPS ${_EDX_CURRENT_MATCH})

			list(TRANSFORM _EDX_PRIVATE_DEPS REPLACE "([^ \t,]+)[ \t]*[><=]*[ \t]*[0-9.]*,?" "\\1")
			list(TRANSFORM _EDX_PRIVATE_DEPS REPLACE "\\\${includedir}" "${_EDX_INCLUDE_DIR}")
			list(TRANSFORM _EDX_PRIVATE_DEPS REPLACE "\\\${libdir}" "${_EDX_LIB_DIR}")
			list(TRANSFORM _EDX_PRIVATE_DEPS REPLACE "\\\${target}" "${_EDX_TARGET}")
			list(APPEND _EDX_DEPS ${_EDX_PRIVATE_DEPS})
		endif()
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^Libs:(.+)$")
		set(_EDX_CURRENT_MATCH ${CMAKE_MATCH_1})
		string(REGEX MATCHALL "-l([^ ]+)" _EDX_PUB_LIBS ${_EDX_CURRENT_MATCH})
		list(TRANSFORM _EDX_PUB_LIBS REPLACE "-l([^ ]+)" "\\1")
		list(FILTER _EDX_PUB_LIBS EXCLUDE REGEX "${_EDX_LIB_FILTER}")

		list(TRANSFORM _EDX_PUB_LIBS REPLACE "\\\${target}" "${_EDX_TARGET}")
		list(TRANSFORM _EDX_PUB_LIBS REPLACE "\\\${prefix}" "${_EDX_PREFIX}")
		list(TRANSFORM _EDX_PUB_LIBS REPLACE "\\\${exec_prefix}" "${_EDX_EXE_PREFIX}")
		if (${_EDX_APR_MAJOR_VERSION})
			list(TRANSFORM _EDX_PUB_LIBS REPLACE "\\\${APR_MAJOR_VERSION}" "${_EDX_APR_MAJOR_VERSION}")
		endif()
		if (${_EDX_APRUTIL_MAJOR_VERSION})
			list(TRANSFORM _EDX_PUB_LIBS REPLACE "\\\${APRUTIL_MAJOR_VERSION}" "${_EDX_APRUTIL_MAJOR_VERSION}")
		endif()

		set(${ARGV1} ${_EDX_PUB_LIBS} PARENT_SCOPE)
		if ("${_EDX_CURRENT_MATCH}" MATCHES ".*-mwindows.*")
			if (MSVC)
				set(${ARGV6} "/link /SUBSYSTEM:windows" PARENT_SCOPE)
			else()
				set(${ARGV5} "-mwindows" PARENT_SCOPE)
			endif()
		endif()
	elseif("${_EDX_CURRENT_LINE}" MATCHES "^Libs.private:(.+)$")
		set(_EDX_CURRENT_MATCH ${CMAKE_MATCH_1})
		string(REGEX MATCHALL "-l([^ ]+)" _EDX_PRIVATE_LIBS ${_EDX_CURRENT_MATCH})
		list(TRANSFORM _EDX_PRIVATE_LIBS REPLACE "-l([^ ]+)" "\\1")
		list(FILTER _EDX_PRIVATE_LIBS EXCLUDE REGEX "${_EDX_LIB_FILTER}")
		set(${ARGV2} ${_EDX_PRIVATE_LIBS} PARENT_SCOPE)
		if ("${_EDX_CURRENT_MATCH}" MATCHES ".*-mwindows.*")
			if (MSVC)
				set(${ARGV6} "/link /SUBSYSTEM:windows" PARENT_SCOPE)
			else()
				set(${ARGV5} "-mwindows" PARENT_SCOPE)
			endif()
		endif()
	endif()
	endforeach()

	list(REMOVE_DUPLICATES _EDX_DEPS)
	set(${ARGV7} ${_EDX_DEPS} PARENT_SCOPE)
endif()
endfunction()

function(edx_download_db_file)
file(DOWNLOAD
	"${_EDX_MSYS_DB_FILE}"
	"${_EDX_MSYS_DB_FILE_CACHE}"
	STATUS _EDX_DOWNLOAD_STATUS
)
if ("0" IN_LIST _EDX_DOWNLOAD_STATUS)
	# download success
	message("Build CMake packages...")
	file(MAKE_DIRECTORY "${_EDX_CMAKE_ROOT}")
	execute_process(COMMAND
		"${_EDX_GEN_MSYS_LIBS}" "${_EDX_MSYS_DB_FILE_CACHE}" "${_EDX_CMAKE_ROOT}"
		WORKING_DIRECTORY "${_EDX_PKG_CACHE}"
	)
endif()
endfunction()

function(edx_update_db_file)
if (NOT EXISTS "${_EDX_MSYS_DB_FILE_CACHE}")
	file(DOWNLOAD
		"${_EDX_MSYS_DB_FILE}.sig"
		"${_EDX_MSYS_DB_FILE_CACHE}.sig"
		STATUS _EDX_DOWNLOAD_STATUS
	)
	message("Download ${_EDX_MSYS_DB_FILE}...")
	edx_download_db_file()
else()
	string(TIMESTAMP _EDX_CURRENT_TIME "%s" UTC)
	file(TIMESTAMP "${_EDX_MSYS_DB_FILE_CACHE}" _EDX_MSYS_DB_MTIME "%s" UTC)
	math(EXPR _EDX_CHECK_INTERVAL "(${_EDX_CURRENT_TIME} - ${_EDX_MSYS_DB_MTIME})/86400")
	if ("${_EDX_CHECK_INTERVAL}" GREATER_EQUAL "1") # check update everyday
		file(DOWNLOAD
			"${_EDX_MSYS_DB_FILE}.sig"
			"${_EDX_MSYS_DB_FILE_CACHE}.sig.new"
			STATUS _EDX_DOWNLOAD_STATUS
		)
		file(MD5 "${_EDX_MSYS_DB_FILE_CACHE}.sig.new" _EDX_NEW_SIG)
		file(MD5 "${_EDX_MSYS_DB_FILE_CACHE}.sig" _EDX_OLD_SIG)
		if (NOT _EDX_OLD_SIG EQUAL _EDX_NEW_SIG)
			message("Update ${_EDX_MSYS_DB_FILE}...")
			file(REMOVE "${_EDX_MSYS_DB_FILE_CACHE}.sig")
			file(RENAME "${_EDX_MSYS_DB_FILE_CACHE}.sig.new" "${_EDX_MSYS_DB_FILE_CACHE}.sig")
			edx_download_db_file()
		else()
			file(REMOVE "${_EDX_MSYS_DB_FILE_CACHE}.sig.new")
			file(TOUCH "${_EDX_MSYS_DB_FILE_CACHE}")
		endif()
	endif()
endif()
endfunction()

#############################################################################

if(NOT DEFINED ENV{_EDX_MSYS_HOST_URL})
	set(_EDX_MSYS_HOST_URL https://mirror.msys2.org/mingw)
else()
	set(_EDX_MSYS_HOST_URL $ENV{_EDX_MSYS_HOST_URL})
endif()

edx_check_target_arch()
if(NOT DEFINED ENV{_EDX_DATA_ROOT})
	file(REAL_PATH "${PROJECT_BINARY_DIR}/../../../msys2/${_EDX_MSYS_CURRENT_HOST}-pkgs" _EDX_PKG_CACHE)
else()
	file(REAL_PATH "$ENV{_EDX_DATA_ROOT}/msys2-cache/${_EDX_MSYS_CURRENT_HOST}" _EDX_PKG_CACHE)
endif()

if(NOT DEFINED ENV{_EDX_HOME})
	set(_EDX_GEN_MSYS_LIBS "gen-msys-libs")
else()
	file(REAL_PATH "$ENV{_EDX_HOME}/gen-msys-libs.exe" _EDX_GEN_MSYS_LIBS)
endif()

file(REAL_PATH "${PROJECT_BINARY_DIR}/../../../msys2" _EDX_PKG_ROOT)
set(_EDX_MSYS_DB_FILE ${_EDX_MSYS_HOST_URL}/${_EDX_MSYS_CURRENT_HOST}/${_EDX_MSYS_CURRENT_HOST}.files.tar.zst)
set(_EDX_MSYS_DB_FILE_CACHE ${_EDX_PKG_CACHE}/${_EDX_MSYS_CURRENT_HOST}.files.tar.zst)
set(_EDX_CMAKE_ROOT ${_EDX_PKG_CACHE}/cmake)
edx_update_db_file()
list(APPEND CMAKE_MODULE_PATH "${_EDX_CMAKE_ROOT}")
