function mgr:on_cxx_completion(doc, status, helper)
	-- C++ 自动补全
	-- status 
	-- 0 popup
	-- 1 commit	with enter
	-- 2 typing
	-- 3 dismiss completion list
	-- 4 dismiss completion tip
	-- 5 item selected
	-- 6 commit with tab
	-- 0x1001 LSP update list
	-- print("complete:", status);
	if status == 3 or status == 4 then
        return 1;
	end;
	local in_completing_state = self.__completing;
	self.__completing = 1;
    -- 得到当前行
	local text_line = doc.get_text(doc.cursor_line, 0, doc.cursor_column);
	local text = text_line;
    if text == nil then
		text = "";
    end;
	local last_cmd = self.__last_cmd;
	-- print("    text:", text);
	-- print_table(last_cmd);
	if text:match("([^ \t]+)") == nil and (status == 2 or status == 0x1001) and last_cmd[1] ~= _op.show_completion_list then
		-- 行内容删空后,自动关闭补全
		self.__completing = nil;
		helper.cancel_completion();
		lsp:cleanup_completion(doc);
        return 0;
	end;
	-- trim front blank chars
	
	text = text:match("^%s*(.*)");

	-- print( "complete ", status, " :", text );
	if doc.cursor_line <= 50 and text:match("^//.*$") ~= nil  then
		self.__completing = 0x010000;	-- completing compile commands
		if status == 0 or in_completing_state == 1 then
			lsp:cleanup_completion(doc);
			helper.clear(false);
			local pp_icon = 0;
			if text:match("^%s*//%s*toolset:.*$") ~= nil then
				self.__completing = self.__completing + 1;	-- completing toolset
				local gcc_icon = icon("gcc");
				local vs_icon = icon("msvc");
				local linux_icon = icon("linux");
				local ssh_icon = icon("ssh");
				local tool_icon = 0;
				for i, toolset in ipairs(TOOLSETS) do
					if toolset["wsl"] then
						tool_icon = linux_icon;
					elseif toolset["remote"] then
						tool_icon = ssh_icon;
					elseif toolset["type"] == "mingw" then
						tool_icon = gcc_icon;
					elseif toolset["type"] == "msvc" then
						tool_icon = vs_icon;
					else
						tool_icon = 0;
					end;
					helper.push_back(toolset.name, tool_icon, 0xFFF0);
				end ;
				status = 0x1001; -- act as LSP
			elseif text:match("^%s*//%s*[^:]*$") ~= nil then
				self.__completing = self.__completing + 2;	-- completing toolset
				helper.push_back("toolset", pp_icon, 0xFFF1);
				helper.push_back("lib", pp_icon, 0xFFF1);
				helper.push_back("lib_path", pp_icon, 0xFFF1);
				helper.push_back("include", pp_icon, 0xFFF1);
				helper.push_back("flags", pp_icon, 0xFFF1);
				helper.push_back("ld_flags", pp_icon, 0xFFF1);
				helper.push_back("args", pp_icon, 0xFFF1);
				helper.push_back("cwd", pp_icon, 0xFFF1);
				helper.push_back("env", pp_icon, 0xFFF1);
				status = 0x1001; -- act as LSP
			else
				helper.cancel_completion();
				return 0;
			end;
		elseif status == 1 then
			local text = doc.get_text(doc.cursor_line, 0, 2000);
			local selected_text = helper.selected_item_text;
			local selected_item_type = helper.selected_item_data;
			if selected_item_type == 0xFFF0 then
				local parts = {text:match("^(%s*//%s*toolset%s*:%s*)(.*)$")};
				doc.select_text(doc.cursor_line, sizeof(parts[1]), doc.cursor_line, sizeof(parts[1]..parts[2]));
				doc.set_text(selected_text);
			elseif selected_item_type == 0xFFF1 then
				local parts = {text:match("^(%s*//%s*)([^:]*:?)(.*)$")};
				doc.select_text(doc.cursor_line, sizeof(parts[1]), doc.cursor_line, sizeof(parts[1]..parts[2]));
				doc.set_text(selected_text..":");
			end;
			helper.cancel_completion();
			lsp:cleanup_completion(doc);
			return 0;
		elseif status == 5 then
			return 1;
		elseif status == 2 then
			status = 0x1001; -- act as LSP
		end;
	elseif text:match("^%s*#%s*include%s+[\"<].-$") ~= nil then
		self.__completing = 0x010100;
		-- 自动补全include, 编辑器内置功能
		lsp:cleanup_completion(doc);
        return 0;
	elseif text:match("^#%s*include%s+$") ~= nil then
        -- 自动补全include 关闭
        helper.cancel_completion();
		return 0;
	elseif text:match("^#.*$") ~= nil then
		self.__completing = 0x010200;
		if status == 5 then
			return 1;
		end;
		if text:match("^#%w+%s+.*$") ~= nil then
			-- complete preprocessor arguments with LSP
		elseif status == 0 or in_completing_state == 1 then
			-- 自动补全预处理符号
			lsp:cleanup_completion(doc);
			helper.clear(false);
			local pp_icon = icon("preprocessor");
			helper.push_back("include", pp_icon, 0);
			helper.push_back("if", pp_icon, 0);
			helper.push_back("ifdef", pp_icon, 0);
			helper.push_back("ifndef", pp_icon, 0);
			helper.push_back("line", pp_icon, 0);
			helper.push_back("else", pp_icon, 0);
			helper.push_back("elif", pp_icon, 0);
			helper.push_back("elifdef", pp_icon, 0);
			helper.push_back("elifndef", pp_icon, 0);
			helper.push_back("endif", pp_icon, 0);
			helper.push_back("define", pp_icon, 0);
			helper.push_back("undef", pp_icon, 0);
			helper.push_back("pragma", pp_icon, 0);
			helper.push_back("error", pp_icon, 0);
			helper.push_back("warning", pp_icon, 0);
			status = 0x1001; -- act as LSP
		elseif status == 2 then
			status = 0x1001; -- act as LSP
		end;
	end;

	if status == 5 then
		lsp:update_completion_tip(doc, helper);
		return 1;
	end;

	local context;
	do
		-- 取得最后一部分
		local last_text = text:match([[.*[-%s@#%$%%%^%&%*%.%(%)!`+=\\/?<>%[%]|:;'",](.-)$]]);
		if last_text ~= nil then
			context = last_text;
		else
			context = text;
		end;
		-- print( "context ", context, " text ", text, " last ", last_text );
	end;

	if status == 1 or status == 6 then
		local commit_mode = 0;
		if status == 6 then
			commit_mode = 1;
		end;
		
		local commit_type = lsp:commit_completion(doc, helper, commit_mode);
		local selected_text = helper.selected_item_text;
		if commit_type and commit_type > 0 then
			if commit_type == 2 then
				-- parameter info required
				edx:do_cmd(_op.show_completion_tip);
			elseif commit_type == 3 then
				local text = doc.get_text(doc.cursor_line, doc.cursor_column, 3);
				if text == "();" then
					doc.set_cursor(doc.cursor_line, doc.cursor_column + 3);
				elseif text:sub(1, 2) == "()" then
					doc.set_cursor(doc.cursor_line, doc.cursor_column + 2);
				elseif text:sub(1, 1) == "(" then
					doc.set_cursor(doc.cursor_line, doc.cursor_column + 1);
				end;
			elseif commit_type == 4 then
				-- keyword
				local text = doc.get_text(doc.cursor_line, doc.cursor_column, 32);
				if text == nil or text == "" or text:match("[0-9A-Za-z_]+.*") then
					-- ...[KEY]...[<EMPTY> or <KEYWORD_CHAR>]
					local prev_text = doc.get_text(doc.cursor_line, 0, doc.cursor_column - #selected_text);
					if prev_text == nil or prev_text == "" or prev_text:match("^%s*$") or prev_text:match("^%s*for%s*%(%s*$") then
						doc.write_char(" ");
					end;
				elseif text:match("[%);, ].*") then
					-- for...(...<KEY>...[);, ]
					local prev_text = doc.get_text(doc.cursor_line, 0, doc.cursor_column - #selected_text);
					if prev_text and prev_text:match("^%s*for%s*%(%s*$") then
						if text == " " then
							doc.set_cursor(doc.cursor_line, doc.cursor_column + 1);
						else
							doc.write_char(" ");
						end;
					end;
				end;
			end;
			return 1;
		end;
		local selected_icon = helper.selected_item_icon;
		local namespace_icon = icon("namespace");
		local method_icon = icon("method");
		local keyword_icon = icon("keyword");
		local pp_icon = icon("preprocessor");
		local update_completion_list = false;

        -- 选择当前项
        local offset = 0;
        while true do
            local c = doc.get_text(doc.cursor_line, doc.cursor_column + offset, 1);
			if c:match("^([#_%w])$") == nil then
                break ;
            end;
            offset = offset + 1;
		end;
		if self.__completing == 0x010200 then
			local file_name = doc.file_name:match("^.*[\\/](.+)");
			if doc.cursor_line == 0 then
				if selected_text == "ifndef" and file_name ~= nil then
					doc.select_text(doc.cursor_line, 0, doc.cursor_line, doc.cursor_column + offset);
					selected_text = "#ifndef";
					file_name = file_name:upper();
					file_name = file_name:gsub("[-=+%.]","_");
					file_name = [[__]]..file_name..[[_]]..os.date([[%Y_%m_%d]])..[[__]];
					doc.replace(selected_text.." ", false);
					selected_text = file_name .. "\n#define " .. file_name .. "\n\n";
					doc.replace(selected_text, false);
					doc.set_cursor(0x7FFFFFFF, 0x7FFFFFFF);
					doc.replace("\n#endif // " .. file_name .. "\n", false);
					doc.set_cursor(2, 0);
					doc.center_cursor_line();

					helper.cancel_completion();
					lsp:cleanup_completion(doc);
					return 1;
				elseif selected_text == "pragma" and file_name ~= nil and (file_name:lower():match(".*%.hh?$") or file_name:lower():match(".*%.hxx$") or file_name:lower():match(".*%.hpp$")) then
					doc.select_text(0, 0, 0, doc.cursor_column + offset);
					doc.set_text("#pragma once");
					helper.cancel_completion();
					lsp:cleanup_completion(doc);
					return 1;
				end;
			end;
			local m = text_line:match("^(%s*#%s*)") or "";
			doc.select_text(doc.cursor_line, #m, doc.cursor_line, doc.cursor_column + offset);
			if selected_text ~= "else" and selected_text ~= "endif" then
				selected_text = selected_text .. " ";
			end;
		else
			doc.select_text(doc.cursor_line, doc.cursor_column - sizeof(context), doc.cursor_line, doc.cursor_column + offset);
			if selected_text == "dynamic_cast"
				or selected_text == "static_cast"
				or selected_text == "reinterpret_cast"
				or selected_text == "const_cast"
			then
				-- c++ cast自动加'<'
				if doc.get_text(doc.cursor_line, doc.cursor_column + offset, 1) == "<" then
					doc.select_text(doc.cursor_line, doc.cursor_column - sizeof(context), doc.cursor_line, doc.cursor_column + offset + 1);
				end;
				selected_text = selected_text .. "<";
			elseif namespace_icon == selected_icon then
				selected_text = selected_text .. "::";
				update_completion_list = true;
			elseif method_icon == selected_icon then
				-- selected_text = selected_text .. "(";
			end;
		end;
		doc.set_text(selected_text);
		if update_completion_list then
			edx.do_cmd(_op.update_completion_list);
		else
			helper.cancel_completion();
			lsp:cleanup_completion(doc);
		end;
        return 1;
    end ;
	if status == 0 then
		lsp:do_completion(doc, nil);
		return -1; -- do not show completion list directly
    end;
	if status == 2 or status == 0x1001 then
		local match_state = 0;
        do
            -- 添加光标后内容, 以实现整词选择
			local tail_text = doc.get_text(doc.cursor_line, doc.cursor_column, 1024);
			if tail_text then
				local tail_part = tail_text:match([[^(.-)[-%s@#%$%%%^%&%*%.%(%)!`+=\\/?<>%[%]|:;'",].*$]]);
				tail_text = tail_part or tail_text;
				match_state = helper.select_item(context .. tail_text);
			end;
		end;
		if match_state < 0 then
			match_state = helper.select_item(context);
		end;
		-- match_state
		-- < 0 无匹配项
		-- 0 有完整匹配
		-- 1 头部匹配
		-- 2 分段匹配(如：GeLi 匹配 GetList)
		-- 3 部分匹配
		-- 4 首字母匹配
		-- 5 头部匹配(忽略大小写)
		-- 6 部分匹配(忽略大小写)
		if match_state ~= 0 and status == 2 then
			-- no full matched, update completion list
			if match_state < 0 then
				helper.set_selection_tip(nil);
			end;
			lsp:do_completion(doc, helper);
		end;
        helper.redraw();
        return 1;
    end;
    return 0;
end;
