__last_toolset = nil;

local function empty_project_configs()
	return {
		[".breakpoints"] = {};
		[".editor"] = {opened={},states={}};
		[".targets"] = {};
		[".build"] = {};
	};
end;

local function handle_msvc_build_result(edx, output_lines, ctx, unmap_path)
	local last_type = ctx.last_type;
	local main_file = ctx.main_file or "";
	local type_id = {
		["error"] = 1;
		["warning"] = 2;
		["note"] = 3;
	};
	output_lines = output_lines:gsub("(\r\n)", "\n");
	for output_line in output_lines:gmatch("([^\n]*)") do
		if output_line ~= "" then
			local result = {output_line:match("^%s*(.-)%((%d+),?(%d*)%)%s*:%s*(%w+)%s*([%w%d]*):%s*(.+)$")};
			local result_type = result[4];
			if result_type == "warning" or result_type == "error" then
				main_file = project:regular_path(result[1]);
				last_type = result_type;
				ctx[result_type] = ctx[result_type] + 1;
			elseif result_type == "note" then
				last_type = "note";
			elseif output_line == "        with" or (last_type == "note" and output_line:match("(%s%s%s%s+)")) then
				result = {
					"", "", "", "", output_line
				};
				last_type = "note";
			else
				main_file = "";
				last_type = nil;
			end;
			if last_type then
				local file_name = project:regular_path(result[1]);
				local line = tonumber(result[2]);
				local column = 0;
				local location;
				if result[3] then
					location = file_name.. ":"..result[2]..":"..result[3]..":";
					column = tonumber(result[3]);
				else
					location = file_name.. "("..result[2]..")";
				end;
				local code = result[5];
				local message = result[6];
				if main_file == file_name and message then
					mgr:add_doc_diagnostic("compiler", file_name, line, column, line, -1, type_id[last_type], message);
					mgr:refresh_doc_diagnostic(file_name);
				end;
				edx:add_build_result(type_id[last_type], main_file, file_name, line, location, code, message);
			end;
		end;
	end;
	ctx.last_type = last_type;
	ctx.main_file = main_file;
end;

local function handle_gcc_build_result(edx, output_lines, ctx, unmap_path)
	local last_type = ctx.last_type;
	local main_file = ctx.main_file or "";
	local type_id = {
		["error"] = 1;
		["warning"] = 2;
		["note"] = 3;
	};
	local function flush_tips(last_main, new_main, new_type)
		if last_main == "" or last_type == nil then
			return;
		end;
		local include_tip = ctx.include_tip;
		local inline_tip = ctx.inline_tip;
		local type_id_val = type_id[last_type];
		if include_tip then
			for idx, info in ipairs(include_tip) do
				local location_base = info.file..":"..info.line;
				local location = location_base..":1:";
				edx:add_build_result(type_id_val, main_file, info.file, tonumber(info.line), location, "", "include from "..location_base);
			end;
		end;
		if inline_tip then
			if include_tip then
				edx:add_build_result(type_id_val, main_file, "", 0, "", "", "----------------------------------------");
			end;
			for idx, info in ipairs(inline_tip) do
				local location = info.file..":"..info.line..":"..info.col..":";
				edx:add_build_result(type_id_val, main_file, info.file, tonumber(info.line), location, "", info.info);
			end;
		end;
		ctx.include_tip = nil;
		ctx.inline_tip = nil;
		main_file = new_main;
		last_type = new_type;
	end;
	for output_line in output_lines:gmatch("([^\n]*)") do
		local file_tip = {output_line:match("^I?n? ?f?i?l?e? ?i?n?c?l?u?d?e?d?%s+from%s+(.*):(%d+)[:,]$")};
		if #file_tip > 0 then
			flush_tips(main_file, "", nil);
			output_line = "";
			ctx.include_tip = ctx.include_tip or {};
			table.insert(ctx.include_tip, {
					["file"] = file_tip[1];
					["line"] = file_tip[2];
			});
		else
			file_tip = {output_line:match("^%s+(inlined%s+from%s+'.*')%s+at%s+(.*):(%d+):(%d+)[:,]$")};
			if #file_tip > 0 then
				flush_tips(main_file, "", nil);
				output_line = "";
				ctx.inline_tip = ctx.inline_tip or {};
				table.insert(ctx.inline_tip, {
						["info"] = file_tip[1];
						["file"] = file_tip[2];
						["line"] = file_tip[3];
						["col"] = file_tip[4];
				});
			end;
		end;
		
		local result = {output_line:match("^%s*(.-):(%d+):(%d+):%s*(%w*%s*[%w%d]*)%s*:%s*(.*)")};
		local result_type = result[4];
		if result_type == "fatal error" then
			result_type = "error";
		end;
		if result_type == "warning" or result_type == "error" then
			flush_tips(main_file, "", nil);
			main_file = project:regular_path(unmap_path(result[1]));
			last_type = result_type;
			ctx[result_type] = ctx[result_type] + 1;
		elseif result_type == "note" then
			last_type = "note";
		elseif (last_type == "note" or last_type == "error" or last_type == "warning")
			and (output_line:match("^(%s%s%s%s+)|") or output_line:match("^(%s+%d+%s+)|"))
		then
			last_type = "note";
			result = {
				"", "", "", "", output_line:match("^%s+%d*%s*|(.*)")
			};
			local diag = ctx.__diagnostic;
			if diag then
				local matched_directive = output_line:match("%s+|%s*(%^~*)");
				if matched_directive and diag["message"] then
					ctx.__diagnostic = nil;
					local begin_col = diag["column"];
					local end_col = begin_col + (#matched_directive);
					local file_name = diag["file"];
					mgr:add_doc_diagnostic("compiler", file_name, diag["line"], begin_col, diag["line"], end_col, diag["type"], diag["message"]);
					mgr:refresh_doc_diagnostic(file_name);
				end;
			end;
		elseif output_line == "" or output_line == nil then
			-- skip the empty line
			output_line = nil;
		else
			flush_tips(main_file, "", nil);
			main_file = "";
			last_type = nil;
		end;
		if last_type and output_line then
			local file_name = result[1];
			file_name = project:regular_path(unmap_path(file_name));
			local line = tonumber(result[2]);
			local column = tonumber(result[3]);
			local location = file_name.. ":"..result[2]..":"..result[3]..":";
			local code = "";
			local message = result[5];
			if last_type ~= "note" and main_file == file_name and message then
				ctx.__diagnostic = {
					["file"] = file_name;
					["line"] = line;
					["column"] = column;
					["message"] = message;
					["type"] = type_id[last_type];
				};
				mgr:add_doc_diagnostic("compiler", file_name, line, column, line, -1, type_id[last_type], message);
				mgr:refresh_doc_diagnostic(file_name);
			end;
			edx:add_build_result(type_id[last_type], main_file, file_name, line, location, code, message);
		end;
	end;
	ctx.last_type = last_type;
	ctx.main_file = main_file;
end;

local function gen_clang_result_handler(result_line_pattern)
	return function(edx, output_lines, ctx, unmap_path)
		local last_type = ctx.last_type;
		local main_file = ctx.main_file or "";
		local type_id = {
			["error"] = 1;
			["warning"] = 2;
			["note"] = 3;
		};
		for output_line in output_lines:gmatch("([^\n\r]*)[\n\r]*") do
			local result = {output_line:match(result_line_pattern)};
			local result_type = result[4];
			if result_type == "warning" or result_type == "error" then
				result[1] = project:regular_path(make_path(unmap_path(result[1])));
				main_file = result[1];
				last_type = result_type;
				ctx[result_type] = ctx[result_type] + 1;
			elseif result_type == "note" then
				last_type = result_type;
			elseif (last_type == "note" or last_type == "error" or last_type == "warning") then
				if output_line:match("(In file included from).*") then
					last_type = "note";
					local ret = {output_line:match("In file included from (.*):(%d+):.*")}
					result = {
						ret[1], ret[2], "", "", "", output_line
					};
					result[1] = project:regular_path(make_path(unmap_path(result[1])));
				elseif output_line:match("(%s%s+.*)") then
					last_type = "note";
					result = {
						"", "", "", "", "", output_line
					};
				else
					main_file = "";
					last_type = nil;
				end;
			else
				main_file = "";
				last_type = nil;
			end;
			if last_type then
				local file_name = result[1];
				local line = tonumber(result[2]);
				local column = tonumber(result[3]);
				local location = "";
				if file_name ~= "" then
					if result[3] ~= "" then
						location = file_name.. ":"..result[2]..":"..result[3]..":";
					else
						location = file_name.. "("..result[2].."):";
					end;
				end;
				local code = result[5];
				local message = result[6];
				if message == nil or file_name == nil or code == nil then
					print("ERROR > ", output_line);
				else
					if main_file == file_name and message then
						mgr:add_doc_diagnostic("compiler", file_name, line, column, line, -1, type_id[last_type], message);
						mgr:refresh_doc_diagnostic(file_name);
					end;
					edx:add_build_result(type_id[last_type], main_file, file_name, line, location, code, message);
				end;
			end;
		end;
		ctx.last_type = last_type;
		ctx.main_file = main_file;
	end;
end;

local handle_clang_build_result = gen_clang_result_handler("^%s*(.-):(%d+):(%d+):%s*(%w*)%s*([%w%d]*):%s*(.*)");

local handle_clang_cl_build_result = gen_clang_result_handler("^%s*(.-)%((%d+),(%d+)%):%s*(%w*)%s*([%w%d]*):%s*(.*)");

local function make_target_name(target_info)
	local name = target_info.name;
	if name:match("%s") then
		return [["]]..name..[["]];
	end;
	return name;
end;

local function wrap_bin_path(path, toolset)
	if toolset.remote or toolset.wsl then
		return path;
	end;
	if path:match("%s+") then
		return [["]]..path..[["]];
	end;
	return path;
end;

project = {
	_home = nil;
	_type = "";
	_last_config = {};
	_toolset_changed_listeners = {};
	_project_changed = false;
	load = function(self, dir)
		local cmake_file = make_path(dir, "CMakeLists.txt");
		local xmake_file = make_path(dir, "xmake.lua");
		if utils:file_time(cmake_file) then
			-- is cmake project
			return self:_cmake_load(dir, cmake_file);
		end;
		if utils:file_time(xmake_file) then
			-- is xmake project
			return self:_xmake_load(dir, xmake_file);
		end;
		return false;
	end;
	unload = function(self)
		if self:is_cmake() then
			self:_cmake_unload();
			return;
		end;
		if self:is_xmake() then
			self:_xmake_unload();
			return;
		end;
	end;
	is_cmake = function(self)
		return self._type == "cmake";
	end;
	is_xmake = function(self)
		return self._type == "xmake";
	end;
	is_esp_idf = function(self)
		return self:is_cmake() and ESP32_IDF_CMAKE_PROJECT;
	end;
	is_bouffalo_sdk = function(self)
		return self:is_cmake() and BL_CMAKE_PROJECT;
	end;
	is_ready = function(self)
		return self._home ~= nil;
	end;
	is_project_script = function(self, filepath)
		if self:is_xmake() then
			return filepath == self._xmake_file;
		end;
		if self:is_cmake() then
			return filepath == self._cmake_file;
		end;
		return false;
	end;
	make_path = function(self, ...)
		if not self:is_ready() then
			return nil;
		end;
		return make_path(self._home, ...);
	end;
	make_rel_path = function(self, path)
		local home_path = self._home;
		if home_path == nil or path:sub(1, #home_path) ~= home_path then
			return nil;
		end;
		local rel_path = path:sub(2 + #home_path);
		rel_path = rel_path:gsub('\\', '/');
		return rel_path;
	end;
	reload_settings = function(self, path)
		-- 更新项目配置脚本
		local config_script = (path and make_path(path, "settings.lua")) or self:make_path("settings.lua");
		if config_script and utils:file_time(config_script) then
			dbg_call(function()
					local breakpoints = project_configs and project_configs[".breakpoints"];
					local extra = project_configs and project_configs[".extra"];
					dofile(utils:convert_string(config_script, "utf8", "acp"));
					if breakpoints then
						-- keeps current breakpoints
						project_configs[".breakpoints"] = breakpoints;
					end;
					if extra then
						project_configs[".extra"] = extra;
					end;
			end, "加载启动脚本出错:");
		end;
	end;
	save_settings = function(self, overwrite)
		if not self:is_ready() or project_configs == nil then
			return;
		end;
		local setting_file_name = self:make_path("settings.lua");
		local setting_file_lines = {};
		local append_line = function(text)
			table.insert(setting_file_lines, text);
		end;

		local original_editor_info = project_configs[".editor"];
		if not overwrite then
			self:reload_settings();
		end;
		project_configs[".editor"] = original_editor_info;

		local function write_item(indent, item, name, is_array)
			indent = indent.."\t";
			local item_value = (type(item) == "table" and item[name]) or item;
			if type(item_value) == "string" then
				if is_array then
					append_line(("%s[[%s]];"):format(indent, item_value));
				else
					append_line(("%s[\"%s\"] = [[%s]];"):format(indent, name, item_value));
				end;
			elseif type(item_value) == "number" then
				if is_array then
					append_line(("%s%d;"):format(indent, item_value));
				else
					append_line(("%s[\"%s\"] = %d;"):format(indent, name, item_value));
				end;
			elseif type(item_value) == "table" then
				if is_array then
					append_line(("%s = {"):format(indent));
				else
					append_line(("%s[\"%s\"] = {"):format(indent, name));
				end;
				if #item_value == 0 then
					for key, val in pairs(item_value) do
						write_item(indent, item_value, key);
					end;
				else
					for idx, val in ipairs(item_value) do
						write_item(indent, item_value, idx, true);
					end;
				end;
				append_line(("%s};"):format(indent));
			end;
		end;

		local formatter = {
			[".targets"] = function(target_info)
				for name, info in pairs(target_info) do
					write_item("\t", target_info, name);
				end;
			end;
			[".build"] = function(build_info)
				local target = (menu_bar.get_cmake_default_target() or {name = ""}).name;
				local toolset = (menu_bar.get_cmake_toolset() or "");
				local config = (menu_bar.get_cmake_config() or "");
				append_line(("\t\tconfig = [[%s]];"):format(config));
				append_line(("\t\ttarget = [[%s]];"):format(target));
				append_line(("\t\ttoolset = [[%s]];"):format(toolset));
			end;
			[".editor"] = function(editor_info)
				append_line("\t\topened = {");
				for i, filename in ipairs(editor_info.opened or {}) do
					append_line(("\t\t\t[[%s]];"):format(filename));
				end;
				append_line("\t\t};");
				append_line("\t\tstates = {");
				for file, states in pairs(editor_info.states or {}) do
					local file_path = self:make_path(file);
					local file_time = utils:file_time(file_path);
					-- trimming status of the file that not exist
					if file_time then
						append_line(("\t\t\t[\"%s\"] = {"):format(file:gsub("\"", "\\\"")));
						for key, value in pairs(states) do
							if key == "cursor" then
								append_line(("\t\t\t\tcursor = {%d;%d};"):format(value[1], value[2]));
							elseif key == "fold" then
								append_line(("\t\t\t\tfold = [[%s]];"):format(value));
							end;
						end;
						append_line("\t\t\t};");
					end;
				end;
				append_line("\t\t};");
			end;
			[".breakpoints"] = function(breakpoints)
				for filename, fileinfo in pairs(breakpoints) do
					append_line(("\t\t[\"%s\"] = {"):format(filename));
					for ln, bkp_info in pairs(fileinfo) do
						append_line(("\t\t\t[%d] = {};"):format(ln));
					end;
					append_line("\t\t};");
				end;
			end;
			[".cmake"] = function(cmake_info)
				for name, info in pairs(cmake_info) do
					write_item("\t", cmake_info, name);
				end;
			end;
			[".xmake"] = function(xmake_info)
				for name, info in pairs(xmake_info) do
					write_item("\t", xmake_info, name);
				end;
			end;
			[".extra"] = function(extra_info)
				for name, info in pairs(extra_info) do
					write_item("\t", extra_info, name);
				end;
			end;
		};

		append_line("project_configs = {");
		local save_seq = {".targets", ".build", ".editor", ".breakpoints", ".cmake", ".xmake", ".extra"};
		for idx, name in ipairs(save_seq) do
			append_line(("\t[\"%s\"] = {"):format(name));
			if formatter[name] and project_configs[name] then
				formatter[name](project_configs[name]);
			end;
			append_line("\t};");
		end;
		append_line("};");

		local of=io.u8_open(setting_file_name, "w+");
		of:write(table.concat(setting_file_lines, "\n"));
		of:close();
	end;
	set_extra_setting = function(self, key, val)
		if project_configs == nil then
			return;
		end;
		local extra = project_configs[".extra"];
		if not extra then
			extra = {};
			project_configs[".extra"] = extra;
		end;
		extra[key] = val;
	end;
	get_extra_setting = function(self, key, def_val)
		if project_configs == nil then
			return;
		end;
		local extra = project_configs[".extra"];
		if not extra then
			extra = {};
			project_configs[".extra"] = extra;
		end;
		if extra[key] == nil then
			extra[key] = def_val;
		else
			def_val = extra[key];
		end;
		return def_val;
	end;
	get_scoped_setting = function(self, ...)
		if project_configs == nil then
			return;
		end;
		local toolset = menu_bar.get_cmake_toolset(true);
		local config = menu_bar.get_cmake_config();
		local scope = config.." - "..toolset.name;
		local setting = project_configs;
		local keys = {...};
		table.insert(keys, scope);
		for idx, key in ipairs(keys) do
			setting = (setting and setting[key]) or nil;
		end;
		return setting;
	end;
	find_build_command = function(self, build_target, skip_current_file)
		if skip_current_file ~= true then
			local cmd = self:_current_doc_find_build_command(build_target);
			if cmd then
				return cmd;
			end;
		end;
		if self:is_cmake() then
			return self:_cmake_find_build_command(build_target);
		end;
		if self:is_xmake() then
			return self:_xmake_find_build_command(build_target);
		end;
	end;
	--[[
	cmd -- find_build_command() result
	build_type -- the build type string. "build", "build_target", "rebuild", "rebuild_target"
	on_complete -- task finish callback
	]]
	gen_do_build_params = function(self, cmd, build_type, on_complete, prepend_cmds, apppend_cmds)
		if cmd["type"] == "cmake" then
			return self:_cmake_gen_do_build_params(cmd, build_type, on_complete, prepend_cmds, apppend_cmds);
		end;
		if cmd["type"] == "direct" then
			return self:_current_doc_gen_do_build_params(cmd, build_type, on_complete, prepend_cmds, apppend_cmds);
		end;
		if cmd["type"] == "xmake" then
			return self:_xmake_gen_do_build_params(cmd, build_type, on_complete, prepend_cmds, apppend_cmds);
		end;
	end;
	refresh_targets = function(self, cmd)
		if cmd["type"] == "cmake" then
			return self:_cmake_reload_project_info(cmd);
		end;
		if cmd["type"] == "native" then
			return nil;
		end;
		if cmd["type"] == "xmake" then
			return self:_xmake_refresh_targets(cmd);
		end;
	end;
	targets = function(self)
		if self:is_cmake() then
			return self._cmake_targets;
		end;
		if self:is_xmake() then
			return self._xmake_targets;
		end;
		return {};
	end;
	fix_path_env = function(self, env)
		local targets = self:targets();
		local paths = env["PATH"] or os.getenv("PATH");
		if paths then
			paths = {split_string(paths, ";")};
		else
			paths = {};
		end;
		local insert_pos = 1;
		for idx, target in ipairs(targets) do
			if (target["type"] == 1 or target["type"] == 3) and target["output"] then
				-- is executable or shared lib
				table.insert(paths, insert_pos, make_path(target["output"], ".."));
				insert_pos = insert_pos + 1;
			end;
		end;
		paths = table.unique(paths);
		env["PATH"] = table.concat(paths, ";");
		return env;
	end;
	store_breakpoints = function(self, doc)
		if not self:is_ready() then
			return;
		end;
		if project_configs == nil then
			project_configs = empty_project_configs();
		end;

		local name = doc.file_name;
		local rel_path = self:make_rel_path(name);
		if rel_path == nil then
			return;
		end;
		local bkp_lines = {doc.get_all_breakpoints()};
		local breakpoints = project_configs[".breakpoints"];
		if breakpoints == nil then
			breakpoints = {};
			project_configs[".breakpoints"] = breakpoints;
		end;
		if #bkp_lines <= 0 then
			breakpoints[rel_path] = nil;
			return;
		end;
		local bkp_info = {};
		breakpoints[rel_path] = bkp_info;
		for i, ln in ipairs(bkp_lines) do
			bkp_info[ln] = {};
		end;
	end;
	store_file_states = function(self, doc)
		if not self:is_ready() then
			return;
		end;
		if project_configs == nil then
			project_configs = empty_project_configs();
		end;

		local name = doc.file_name;
		local rel_path = self:make_rel_path(name);
		if rel_path == nil then
			return;
		end;
		local editor = project_configs[".editor"];
		if editor == nil then
			editor = {opened={};states={}};
			project_configs[".editor"] = editor;
		end;
		editor.states[rel_path] = {
			cursor = {doc.cursor_line;doc.cursor_column;};
			fold = doc.fold_status;
		};
	end;
	restore_breakpoints = function(self, doc)
		if not self:is_ready() or project_configs == nil then
			return;
		end;

		local name = doc.file_name;
		local rel_path = self:make_rel_path(name);
		if rel_path == nil then
			return;
		end;
		local breakpoints = project_configs[".breakpoints"] or {};
		local bkp_infos = breakpoints[rel_path];
		if bkp_infos == nil then
			return;
		end;
		for ln, info in pairs(bkp_infos) do
			doc:toggle_breakpoint(ln);
		end;
	end;
	restore_file_states = function(self, doc)
		if not self:is_ready() or project_configs == nil then
			return;
		end;

		local name = doc.file_name;
		local rel_path = self:make_rel_path(name);
		if rel_path == nil then
			return;
		end;
		local editor = project_configs[".editor"] or {states={}};
		local states = editor.states[rel_path] or {cursor={0;0}};
		local line = states.cursor[1];
		local column = states.cursor[2];
		doc.set_cursor(line, column);
		if states.fold then
			doc.fold_status = states.fold;
		end;
		doc.center_cursor_line();
	end;
	schedule_reconfig = function(self)
		self._last_config = {};
	end;
	on_toolset_changed = function(self, toolset)
		local show_items = {};
		local hide_items = {};
		-- "toolset",
		-- "serial_ports",
		-- "serial_baud",
		-- "cpus",
		-- "debug_adapters"
		-- "boards",
		local make_toolset_menu = function(items, key, name)
			menu_bar["update_cmake_"..key](items);
			if #items > 0 then
				table.insert(show_items, name);
			else
				table.insert(hide_items, name);
			end;
		end;
		if toolset.debug_adapter then
			debug_adapter = toolset.debug_adapter;
			local adapters = table.keys(debug_adapter.adapters or {});
			table.sort(adapters);
			make_toolset_menu(adapters, "debug_adapter", "debug_adapters");
			debug_adapter:switch_adapter(menu_bar.get_cmake_debug_adapter());
			make_toolset_menu(debug_adapter.transports, "debug_adapter_transport", "debug_adapter_transport");
			make_toolset_menu(debug_adapter.speeds, "debug_adapter_speed", "debug_adapter_speed");
		else
			hide_items = {"debug_adapters", "debug_adapter_speed", "debug_adapter_transport"};
		end;
		if #show_items > 0 then
			menu_bar.show_cmake_items(table.unpack(show_items));
		end;
		if #hide_items > 0 then
			menu_bar.hide_cmake_items(table.unpack(hide_items));
		end;
		for idx, listener in ipairs(self._toolset_changed_listeners) do
			pinvoke(listener, toolset);
		end;
	end;
	on_file_renamed = function(self, old_name, new_name)
		if not self:is_ready() then
			return;
		end;
		self._project_changed = true;
	end;
	on_file_removed = function(self, file_name)
		if not self:is_ready() then
			return;
		end;
		self._project_changed = true;
	end;
	on_file_created = function(self, file_name)
		if not self:is_ready() then
			return;
		end;
		self._project_changed = true;
	end;
	find_debug_target = function(self, toolset, debug_adapter_name)
		if self:is_esp_idf() then
			return toolset.target, toolset.target;
		end;
		local debug_adapter = toolset.debug_adapter;
		local cmake_file = self._cmake_file;
		if not debug_adapter or not cmake_file then
			return;
		end;
		local debug_adapter_info = debug_adapter.adapters[debug_adapter_name];
		local debug_target;
		if not debug_adapter_info then
			return;
		end;
		
		if toolset["type"] == "PicoSDK" then
			if debug_adapter_info["type"] == "jlink" then
				debug_target = "rp2040_m0_0";
			else
				debug_target = "rp2040";
			end;
		else
			do
				local f = io.u8_open(cmake_file, "r");
				if f then
					for line in f:lines() do
						local lower_line = line:lower();
						debug_target = lower_line:match("set%(%s*target_cpu%s+(.-)%s*%)");
						if debug_target then
							break;
						end;
					end;
					f:close();
				end;
			end;
		end;
		local orig_debug_target = debug_target;
		if debug_target == "air001" then
			if debug_adapter_info["type"] == "jlink" then
				debug_target = "air001dev";
			end;
		elseif debug_target:sub(1,5) == "air32" then
			if debug_adapter_info["type"] == "jlink"
				and
				utils:file_time(make_path(os.getenv("APPDATA"), [[SEGGER\JLinkDevices\AirM2M\Air32\Devices.xml]])) ~= nil
			then
				-- AirM2M devices installed
			else
				-- use stm32 instead
				local air32device_map = {
					["air32f103cb"] = "stm32f103rb";
					["air32f103cc"] = "stm32f103rc";
				};
				debug_target = air32device_map[debug_target] or "stm32f103rb";
			end;
		end;

		if debug_adapter_info["type"] == "openocd" then
			if debug_target:sub(1,5) == "stm32" then
				-- fix stm32 targets
				local stm32prefix, series = debug_target:match("^stm32(%w-)([0-9]+.*)");
				debug_target = "stm32"..stm32prefix;
				if stm32prefix == "l" then
					if series:sub(1,1) == "1" then
						debug_target = debug_target.."1";
					elseif series:sub(1,1) == "0" then
						debug_target = debug_target.."0";
					else
						debug_target = debug_target..series:sub(1,1).."x";
					end;
				elseif stm32prefix == "mp" then
					if series:sub(1,2) == "13" then
						debug_target = debug_target.."13x";
					elseif series:sub(1,1) == "15" then
						debug_target = debug_target.."15x";
					else
						debug_target = debug_target..series:sub(1,1).."x";
					end;
				else
					debug_target = debug_target..series:sub(1,1).."x";
				end;
			elseif debug_target:sub(1,4) == "gd32" then
				local gd32series, ext = debug_target:match("^gd32(%w%d%d%d)(.*)");
				if gd32series == "a490" or
					gd32series == "a508" or
					gd32series == "e232" or
					gd32series == "e501" or
					gd32series == "e502" or
					gd32series == "f403"
				then
					debug_target = "gd32"..gd32series;
				elseif gd32series:match("^f[13]%d0") then
					debug_target = "gd32"..gd32series:sub(1, 2).."x"..gd32series:sub(4,4);
				elseif gd32series:match("^f[45]%d%d") or gd32series:match("^h7%d%d") then
					debug_target = "gd32"..gd32series:sub(1, 2).."xx";
				else
					debug_target = "gd32"..gd32series:sub(1, 3).."x";
				end;
			elseif debug_target:match("^cc2538.*") then
				debug_target = "cc2538";
			elseif debug_target:match("^cc3220sf.*") then
				debug_target = "ti_cc3220sf";
			elseif debug_target:match("^cc32%d%d.*") then
				debug_target = "ti_cc32xx";
			elseif debug_target:match("^cc26%d0.*") then
				debug_target = "ti_cc26x0";
			elseif debug_target:match("^cc13%d0.*") then
				debug_target = "ti_cc13x0";
			elseif debug_target:match("^cc26%d2.*") then
				debug_target = "ti_cc26x2";
			elseif debug_target:match("^cc13%d2.*") then
				debug_target = "ti_cc13x2";
			end;
		elseif debug_adapter_info["type"] == "pyocd" then
			if debug_target:sub(1,5) == "stm32" then
				-- fix stm32 targets
				local stm32prefix, series = debug_target:match("^stm32(%w-)([0-9]+.*)");
				if stm32prefix == "h" then
					local series_map = {
						["74"] = "stm32h743xx";
						["75"] = "stm32h743xx";
						["72"] = "stm32h723xx";
						["73"] = "stm32h723xx";
						["7a"] = "stm32h7b0xx";
						["7b"] = "stm32h7b0xx";
					};
					debug_target = series_map[series:sub(1,2)] or debug_target;
				end;
			end;
		end;
		return debug_target, orig_debug_target;
	end;
	_find_result_handler = function(self, toolset)
		local result_handler = handle_msvc_build_result;
		if toolset["type"] == "mingw" or
			toolset["type"] == "gcc" or
			toolset["type"] == "bouffalo_sdk" or
			toolset["type"] == "esp-idf"
		then
			result_handler = handle_gcc_build_result;
		elseif toolset["type"] == "clang-cl" then
			result_handler = handle_clang_cl_build_result;
		elseif toolset["type"] == "clang-msvc" or
			toolset["type"] == "clang-mingw" or
			toolset["type"] == "llvm-mingw" or
			toolset["type"] == "llvm-msys2" or
			toolset["type"] == "clang"
		then
			result_handler = handle_clang_build_result;
		elseif toolset["type"] == "msvc" then
			result_handler = handle_msvc_build_result;
		end;
		return result_handler;
	end;
	_fix_toolset_extra_env = function(self, toolset_extra_envs, toolset, cmd)
		if toolset["type"] == "mingw" or
			toolset["type"] == "clang-mingw" or
			toolset["type"] == "llvm-mingw" or
			toolset["type"] == "llvm-msys2"
		then
			local concat_args = function(arg_table, arg_seperator, prefix_mode)
				if not arg_table or type(arg_table) ~= "table" or #arg_table <= 0 then
					return nil;
				end;
				if prefix_mode then
					local args = {};
					for idx, val in ipairs(arg_table) do
						table.insert(args, arg_seperator.."\""..val.."\"");
					end;
					return table.concat(args, "");
				end;
				return table.concat({table.unpack(arg_table)}, arg_seperator);
			end;
			toolset_extra_envs["CXXFLAGS"] = concat_args(cmd.cmake_inc_path, " -I", true);
			toolset_extra_envs["LDFLAGS"] = concat_args(cmd.cmake_lib_path, " -L", true);
			toolset_extra_envs["CFLAGS"] = toolset_extra_envs["CXXFLAGS"];
			toolset_extra_envs["CMAKE_LIBRARY_PATH"] = concat_args(cmd.cmake_lib_path, ";");
		elseif toolset["type"] == "msvc" or
			toolset["type"] == "clang-msvc" or
			toolset["type"] == "clang-cl"
		then
			toolset_extra_envs["INCLUDE"] = table.concat(table.iclone(toolset.include, cmd.cmake_inc_path), ";");
			toolset_extra_envs["LIB"] = table.concat(table.iclone(toolset.lib, cmd.cmake_lib_path), ";");
		elseif toolset["target"] == "PICO" then
			for idx, toolset in ipairs(TOOLSETS) do
				if toolset.name:match("^vs%d%d%d%d%-.*") then
					toolset_extra_envs["INCLUDE"] = table.concat(table.iclone(toolset.include, cmd.cmake_inc_path), ";");
					toolset_extra_envs["LIB"] = table.concat(table.iclone(toolset.lib, cmd.cmake_lib_path), ";");
					break;
				end;
			end;
		end;
	end;
	_find_target_config = function(self, toolset, target_name, toolset_name, config, make_bin_path)
		local target_cwd = nil;
		local target_argument = nil;
		local target_environment = nil;
		if project_configs ~= nil then
			local user_config = (project_configs[".targets"] and project_configs[".targets"][target_name]) or {};
			local config_type = toolset_name.."-"..config;
			local get_config_setting = function(settings)
				if type(settings) == "table" then
					return settings[config_type] or settings["default"];
				elseif type(settings) == "string" then
					return settings;
				end;
				return nil;
			end;
			local cwd = get_config_setting(user_config.cwd);
			local map_path = toolset.map_path or function(path) return path; end;
			if cwd then
				if toolset and toolset.wsl then
					target_cwd = cwd;
				elseif cwd:match([[^%w:\.*$]]) then
					target_cwd = cwd;
				else
					target_cwd = self:make_path(cwd);
				end;
			else
				-- print("Can't find target_cwd for config ", config_type);
			end;
			target_argument = get_config_setting(user_config.argument);
			target_environment = get_config_setting(user_config.environment);
		end;
		if toolset.bin and not (toolset.wsl or toolset.remote) then
			if target_environment == nil then
				target_environment = {};
			end;
			local sys_path = table.iclone(
				{split_string((target_environment["PATH"] or ""), ";")},
				toolset.bin,
				toolset.lib,
				make_bin_path,
				{split_string((os.getenv("PATH") or ""), ";")}
			);
			target_environment["PATH"] = table.concat(table.unique(sys_path), ";");
		end;
		if not target_cwd then
			target_cwd = self._home;
		end;
		return target_cwd, target_argument, target_environment;
	end;
	_find_target_info = function(self, target_name)
		if not target_name then
			return nil;
		end;
		for idx, info in ipairs(self:targets()) do
			if target_name == info.name then
				return info;
			end;
		end;
		return nil;
	end;
	_fix_compile_commands_json = function(self, cmd)
		local toolset = cmd.toolset;
		if not toolset.wsl and not toolset.remote and toolset["type"] ~= "esp-idf" and toolset["type"] ~= "bouffalo_sdk" then
			return;
		end;

		local build_path = cmd.build_path;
		local unmap_path = toolset.unmap_path or function(path) return path; end;
		local org_file_name = make_path(build_path, "/compile_commands.json");
		local new_file_name = org_file_name..".org";
		os.rename(utils:convert_string(org_file_name, "utf8", "acp"), utils:convert_string(new_file_name, "utf8", "acp"));
		local org_file = io.u8_open(new_file_name);
		if toolset.wsl then
			-- remap wsl path for compile_commands.json
			local query_driver = make_path(build_path, "../wsl-"..toolset.wsl.."-gcc.exe");
			query_driver = query_driver:gsub("\\", "\\\\");
			if org_file ~= nil then
				local mapped_file = io.u8_open(org_file_name, "w+");
				for ln in org_file:lines() do
					ln = string.gsub(ln, "-I/mnt/(%w)/", "-I%1:/");
					ln = string.gsub(ln, "-c /mnt/(%w)/", "-c %1:/");
					ln = string.gsub(ln, "\"/mnt/(%w)/", "\"%1:/");
					ln = string.gsub(ln, "\"/[^/]+/.*/g[%+c][%+c]", "\""..query_driver);
					ln = string.gsub(ln, "\"g[%+c][%+c]\"", "\""..query_driver.."\"");
					ln = string.gsub(ln, "-isystem (/[%w/%-_%./]*)", function(path)
							return [[-isystem ]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					ln = string.gsub(ln, "-I(/[%w/%-_%./]*)", function(path)
							return [[-I]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					mapped_file:write(ln, "\n");
				end;
				org_file:close();
				mapped_file:close();
				os.remove(utils:convert_string(new_file_name, "utf8", "acp"));
			end;
		elseif toolset.remote then
			-- remap host path for compile_commands.json
			local query_driver = make_path(build_path, "../ssh-gcc.exe");
			query_driver = query_driver:gsub("\\", "\\\\");
			if org_file ~= nil then
				local mapped_file = io.u8_open(org_file_name, "w+");
				for ln in org_file:lines() do
					ln = string.gsub(ln, "-isystem%s*(/[%w/%-_%./]*)", function(path)
							return [[-isystem ]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					ln = string.gsub(ln, "-isystem\"%s*,%s*\"(/[%w/%-_%./]*)", function(path)
							return [[-isystem", "]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					ln = string.gsub(ln, "-I%s*(/[%w/%-_%./]*)", function(path)
							return [[-I]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					ln = string.gsub(ln, "-c (/[%w/%-_%./]*)", function(path)
							return [[-c ]]..(string.gsub(unmap_path(path), "\\", "\\\\"));
					end);
					ln = string.gsub(ln, "\"command\":%s+\"(/%S+)%s", function(path)
							return [["command": "]]..query_driver..[[ ]];
					end);
					ln = string.gsub(ln, "\"arguments\":%s+%[\"(/%S-)\", ", function(path)
							return [["arguments": ["]]..query_driver..[[", ]];
					end);
					ln = string.gsub(ln, "\"file\":%s+\"(.+)\"", function(path)
							return [["file": "]]..(string.gsub(unmap_path(path), "\\", "\\\\"))..[["]];
					end);
					ln = string.gsub(ln, "\"directory\":%s+\"(.+)\"", function(path)
							return [["directory": "]]..(string.gsub(unmap_path(path), "\\", "\\\\"))..[["]];
					end);
					mapped_file:write(ln, "\n");
				end;
				org_file:close();
				mapped_file:close();
				os.remove(utils:convert_string(new_file_name, "utf8", "acp"));
			end;
		elseif toolset["type"] == "esp-idf" then
			-- remap esp path for compile_commands.json
			local query_driver = make_path(build_path, "../esp32-none-gcc.exe");
			query_driver = query_driver:gsub("\\", "\\\\");
			if org_file ~= nil then
				local mapped_file = io.u8_open(org_file_name, "w+");
				local unmap_path = toolset.unmap_path or function(path) return path; end;
				for ln in org_file:lines() do
					ln = string.gsub(ln, [[command":%s*".-\\.-%-g[%+c][%+c]%.exe]], "command\": \""..query_driver.. [[ ]]..toolset["define"]);
					ln = string.gsub(ln, "\"file\":%s+\"(.+)\"", function(path)
							return [["file": "]]..(string.gsub(unmap_path(path), [[[/\]+]], [[\\]]))..[["]];
					end);
					ln = string.gsub(ln, "\"directory\":%s+\"(.+)\"", function(path)
							return [["directory": "]]..(string.gsub(unmap_path(path), [[[/\]+]], [[\\]]))..[["]];
					end);
					mapped_file:write(ln, "\n");
				end;
				org_file:close();
				mapped_file:close();
				os.remove(utils:convert_string(new_file_name, "utf8", "acp"));
			end;
		elseif toolset["type"] == "bouffalo_sdk" then
			if org_file ~= nil then
				local mapped_file = io.u8_open(org_file_name, "w+");
				local unmap_path = toolset.unmap_path or function(path) return path; end;
				for ln in org_file:lines() do
					ln = string.gsub(ln, [[-mtune=%w+]], "");
					mapped_file:write(ln, "\n");
				end;
				org_file:close();
				mapped_file:close();
				os.remove(utils:convert_string(new_file_name, "utf8", "acp"));
			end;
		end;
	end;
	_cmake_load = function(self, dir, cmake_file)
		self._home = dir;
		self._type = "cmake";
		self._last_config = {};
		self._cmake_file = cmake_file;
		menu_bar:show_cmake_menu();
		menu_bar:show_debug_menu();
		self:reload_settings();

		local build = project_configs and project_configs[".build"];
		if build and build.config then
			menu_bar.set_cmake_config(build.config);
		end;
		if build and build.toolset then
			menu_bar.set_cmake_toolset(build.toolset);
		end;
		local toolset = menu_bar.get_cmake_toolset(true);
		if (not toolset) or toolset["target"] ~= "wsl" then
			if #CMAKE_PATHS == 0 then
				print("\x1b[33;41m", LANG("CMake not found!"), "\x1b[0m");
				print("\x1b[33;41m", LANG("Download from https://cmake.org/download"), "\x1b[0m");
				return false;
			end;
			if #NINJA_PATHS == 0 then
				print("\x1b[33;41m", LANG("Ninja-Build not found!"), "\x1b[0m");
				print("\x1b[33;41m", LANG("Download from https://ninja-build.org"), "\x1b[0m");
			end;
		end;
		if toolset then
			self:on_toolset_changed(toolset);
		end;
		local save_project_config = false;

		if project_configs == nil then
			project_configs = empty_project_configs();
			save_project_config = true;
		end;

		-- load cmake targets
		local cmd = self:_cmake_find_build_command(nil);
		if cmd ~= nil and cmd.type == "cmake" then
			local do_build_params = self:_cmake_gen_do_build_params(cmd, nil,function()
					local build = project_configs[".build"];
					if build and build.target then
						menu_bar.set_cmake_default_target(build.target);
						xws_mgr:update_targets({menu_bar.get_cmake_default_target()});
					end;
					lsp:reset(cmd);
					cmake_update_include_path_for_opened_documents();

					if save_project_config then
						self:save_settings();
					end;
					-- refresh document structure
					lsp:list_doc_structure(mgr.current_document, nil);
			end);
			self._last_config[cmd.build_path] = cmd.config;
			toolchain_update_include_path_for_opened_documents(); -- use toolchain include path by default
			edx.do_build(do_build_params);
			__last_toolset = cmd.toolset;
			local editor = project_configs and project_configs[".editor"];
			if editor and editor.opened then
				local paths = {};
				for i,filename in ipairs(editor.opened) do
					table.insert(paths, self:make_path(filename));
				end;
				mgr.open_doc(paths);
			end;
		end;

		return true;
	end;
	_cmake_unload = function(self)
		if project_configs == nil then
			project_configs = empty_project_configs();
		end;
		-- close all opened project files
		local all_doc = {mgr.get_all_documents()};
		local all_doc_name = {};
		local all_doc_states = project_configs[".editor"].states or {};
		for idx, doc in ipairs(all_doc) do
			local name = doc.file_name;
			local rel_path = self:make_rel_path(name);
			if rel_path ~= nil then
				table.insert(all_doc_name, rel_path);
				all_doc_states[rel_path] = {
					cursor = {doc.cursor_line;doc.cursor_column;};
					fold = doc.fold_status;
				};
			end;
		end;
		mgr.close_doc(all_doc);
		project_configs[".editor"] = {
			opened = all_doc_name;
			states = all_doc_states;
		};
		self:save_settings();
		self._cmake_targets = nil;
		self._cmake_file = nil;
		self._type = nil;
		self._home = nil;
		project_configs = nil;
	end;
	_cmake_project_target_filter = function(self, target, cmd)
		return false;
	end;
	_cmake_reload_project_info = function(self, cmd)
		local toolset = cmd.toolset;
		local cmake_targets;
		local index_pattern = make_path(cmd.build_path, ".cmake/api/v1/reply/index-*.json");
		local fapi_reply_index = utils:list_dir(index_pattern, true) or {};
		if true and #fapi_reply_index > 0 then
			-- handle cmake file api reply
			local all_valid_files = {};
			local fapi_replay_root = make_path(cmd.build_path, ".cmake/api/v1/reply");
			local valid_idx = fapi_reply_index[#fapi_reply_index];
			for idx=1,#fapi_reply_index-1,1 do
				os.remove(make_path(fapi_replay_root, fapi_reply_index[idx]));
			end;
			local index_json = load_json(make_path(fapi_replay_root, valid_idx));
			local codemodel_file = index_json["reply"]["codemodel-v2"]["jsonFile"];
			local codemodel_json = load_json(make_path(fapi_replay_root, codemodel_file));
			local codemodel_targets = codemodel_json["configurations"][1]["targets"];
			local codemodel_directories = codemodel_json["configurations"][1]["directories"];
			local build_root = codemodel_json["paths"]["build"];

			all_valid_files[valid_idx] = true;
			all_valid_files[codemodel_file] = true;

			local targets = {};
			cmake_targets = targets;
			local target_type = function(t)
				local type_array = {
					["EXECUTABLE"] = 1,
					["STATIC_LIBRARY"] = 2,
					["SHARED_LIBRARY"] = 3,
					["UTILITY"] = 4,
				};
				return type_array[t] or 4;
			end;
			local has_install = false;
			local parse_target = function(target_json)
				if target_json == nil or not target_json["artifacts"] then
					return;
				end;
				if #target_json["artifacts"] == 0 then
					return;
				end;
				local target_path_name = (target_json["artifacts"][1] or {})["path"];
				if target_json["install"] ~= nil then
					has_install = true;
				end;
				local include = {};
				for idx, cgroup in ipairs(target_json["compileGroups"] or {}) do
					local gincs = cgroup["includes"] or {};
					for idx, inc in ipairs(gincs) do
						local path = inc["path"];
						path = path:gsub("\\", "/");
						path = path:gsub("/%.$", "");
						path = path:gsub("/%./", "/");
						table.insert(include, path);
					end;
				end;
				include = table.iclone(include, toolset.include);
				if target_path_name:sub(1,1) == "/" or target_path_name:sub(2,2) == ":" then
					-- absolute path
				else
					target_path_name = build_root.."/"..target_path_name;
				end;
				table.insert(targets, {
						["name"] = target_json["name"];
						["type"] = target_type(target_json["type"]);
						["include"] = table.unique(include);
						["output"] = target_path_name
				});
			end;
			for idx, target in ipairs(codemodel_targets) do
				local target_file = target["jsonFile"];
				target["dir"] = (codemodel_directories[target["directoryIndex"] + 1] or {})["build"];
				if not self:_cmake_project_target_filter(target, cmd) then
					local target_json = load_json(make_path(fapi_replay_root, target_file));
					all_valid_files[target_file] = true;
					parse_target(target_json);
				end;
			end;
			if has_install then
				table.insert(targets, {
						["name"] = "install";
						["type"] = 4;
				});
			end;

			if toolset.remote then
				for idx, name in ipairs(utils:list_dir(fapi_replay_root, true) or {}) do
					if all_valid_files[name] == nil then
						os.remove(make_path(fapi_replay_root, name));
					end;
				end;
			end;
		else
			local cbp_file = utils:list_dir(make_path(cmd.build_path, "*.cbp"));
			cbp_file = make_path(cmd.build_path, cbp_file);
			local cbp_targets = utils:load_cbp(cbp_file);
			cmake_targets = cbp_targets
		end;

		-- fix mapped include path
		if toolset and toolset.unmap_path then
			local unmap_path = toolset.unmap_path;
			for idx, target in ipairs(cmake_targets) do
				if target and type(target.include) == "table" then
					for n, path in ipairs(target.include) do
						target.include[n] = unmap_path(path);
					end;
				end;
			end;
		elseif cmd.cmake_inc_path and #cmd.cmake_inc_path > 0 then
			local extra_inc_path = table.concat(cmd.cmake_inc_path, ";");
			for idx, target in ipairs(cmake_targets) do
				if target and type(target.include) == "table" then
					for n, path in ipairs(target.include) do
						target.include[n] = path .. ";".. extra_inc_path;
					end;
				end;
			end;
		end;
		table.sort(cmake_targets,
			function(a,b)
				return a.name < b.name;
			end
		);
		self._cmake_targets = self:_cmake_project_filter(cmake_targets, cmd);
		xws_mgr:update_targets(self._cmake_targets, true);
		menu_bar.update_cmake_target(self._cmake_targets);
	end;
	_cmake_project_filter = function(self, targets, cmd)
		if cmd.toolset["type"] == "PicoSDK" then
			local new_targets = {};
			for idx, t in ipairs(targets) do
				if t.name ~= "bs2_default" then
					table.insert(new_targets, t);
				end;
			end;
			return new_targets;
		end;
		return targets;
	end;
	_cmake_find_build_command = function(self, build_target)
		local toolset = menu_bar.get_cmake_toolset(true);
		local config = menu_bar.get_cmake_config();
		return self:_cmake_find_build_command_impl(build_target, toolset, config);
	end;
	_cmake_find_build_command_impl = function(self, build_target, toolset, config, cmake_extra_arguments)
		if toolset == nil then
			return nil;
		end;
		if build_target == ".rebuild_cache" then
			build_target = nil;
		end;
		local cmake_cxx = toolset.tools.cxx and string.gsub(toolset.tools.cxx, "\\", "/");
		local cmake_c = toolset.tools.cc and string.gsub(toolset.tools.cc, "\\", "/");
		local cmake_rc = toolset.tools.rc and string.gsub(toolset.tools.rc, "\\", "/");
		local cmake_rc_path = "";
		if cmake_rc ~= nil then
			cmake_rc_path = string.match(cmake_rc, "([^;]+)");
			cmake_rc = string.gsub(cmake_rc, ";", [[" -DCMAKE_RC_FLAGS="]]);
		end;
		local toolset_name = string.gsub(toolset.name, "[:/\\,*?<>'\"]+", "_");
		local cmake_build_path = self:make_path(".edx/cmake/" .. toolset_name .. "/" .. config);
		local cmake_exe = [[cmake]];
		if not cmake_extra_arguments then
			cmake_extra_arguments = ""
		end;
		local ccache_exe;
		if not toolset.remote then
			ccache_exe = CCACHE_PATH;
		end;

		local target_name=toolset["target"] or "";
		if not toolset.remote and (
				target_name == "avr" or
				string.match(target_name, "^arm%-none%-eabi.*$") or
				string.match(target_name, "^aarch64%-none%-.*$") or
				string.match(target_name, "^mips.*%-none%-.*$") or
				string.match(target_name, "^riscv.*%-esp.*$") or
				string.match(target_name, "^xtensa%-.*$")
			)
		then
			cmake_extra_arguments = cmake_extra_arguments.." -DCMAKE_SYSTEM_NAME=\""..toolset["target"].."\""
		elseif target_name == "PICO" then
			cmake_cxx = nil;
			cmake_c = nil;
			-- cmake_extra_arguments = cmake_extra_arguments.." -DCMAKE_CXX_FLAGS=\"-ftabstop=1\" -DCMAKE_C_FLAGS=\"-ftabstop=1\"";
			local pico_compiler_types = {
				["gcc-arm-none-eabi"] = "pico_arm_gcc";
				-- ["clang-arm-none-eabi"] = "pico_arm_clang_arm";
				["clang-arm-none-eabi"] = "pico_arm_clang";
			};
			local compiler = toolset.vers_info[toolset.version];
			cmake_extra_arguments = cmake_extra_arguments.." -DPICO_COMPILER="..pico_compiler_types[compiler["type"]];
		end;

		if toolset.tools and toolset.tools.ccache then
			ccache_exe = toolset.tools.ccache;
		end;
		if ccache_exe and #ccache_exe > 0 then
			ccache_exe = string.gsub(ccache_exe, "\\", "/");
			cmake_extra_arguments = cmake_extra_arguments..
			[[ -DCMAKE_CXX_COMPILER_LAUNCHER="]]..ccache_exe..
			[[" -DCMAKE_CXX_COMPILER_LAUNCHER="]]..ccache_exe..
			[["]];
		end;

		if toolset.tools and toolset.tools.cmake then
			cmake_exe = toolset.tools.cmake;
		elseif not toolset.remote and CMAKE_PATHS and CMAKE_PATHS[1] then
			cmake_exe = CMAKE_PATHS[1];
		end;

		if project_configs and project_configs[".cmake"] then
			local cmake_config_info = project_configs[".cmake"];
			if cmake_config_info.path then
				if utils:file_time(cmake_config_info.path) == nil then
					-- print("user specified cmake [".. cmake_config_info.path.."] is invalid!");
				else
					cmake_exe = cmake_config_info.path;
				end;
			end;
			local scope = config.." - "..toolset.name;
			if cmake_config_info.arguments and cmake_config_info.arguments[scope] then
				cmake_extra_arguments = cmake_extra_arguments.." " .. table.concat(cmake_config_info.arguments[scope], " ");
			end;
			if type(cmake_config_info.argument) == "string" then
				cmake_extra_arguments = cmake_extra_arguments.." " .. project_configs[".cmake"].argument;
			elseif type(cmake_config_info.argument) == "table" then
				cmake_extra_arguments = cmake_extra_arguments.." " .. table.concat(project_configs[".cmake"].argument, " ");
			end;
		end;

		local cmake_bin_path = {};
		local cmake_inc_path = {};
		local cmake_lib_path = {};

		if toolset.tools["vcpkg"] ~= "DISABLED" and VCPKG_PATH then
			local vcpkg_target_triplet = nil;
			if toolset.name:match("^.*-mgw64-.*-m32$") then
				-- mingw x86
				vcpkg_target_triplet = "x86-mingw-dynamic";
			elseif toolset.name:match("^.*-msys-mgw32-.*$") then
				-- mingw/msys x86
				vcpkg_target_triplet = "x86-mingw-dynamic";
			elseif toolset.name:match("^.*-msys-mgw64-.*$") then
				-- mingw/msys x64
				vcpkg_target_triplet = "x64-mingw-dynamic";
			elseif toolset.name:match("^.*-mgw64-.*$") then
				-- mingw x64
				vcpkg_target_triplet = "x64-mingw-dynamic";
			elseif toolset.name:match("^vs%d+-x86$") then
				-- msvc x86
				vcpkg_target_triplet = "x86-windows";
			elseif toolset.name:match("^vs%d+-x64$") then
				-- msvc x64
				vcpkg_target_triplet = "x64-windows";
			end;
			local vcpkg_install_root = nil;
			if VCPKG_TOOLCHAIN_FILE and utils:file_time(self:make_path("vcpkg.json")) then
				-- add vcpkg parameters
				cmake_extra_arguments = cmake_extra_arguments.. [[ -DCMAKE_TOOLCHAIN_FILE="]]..VCPKG_TOOLCHAIN_FILE..[["]];
				if vcpkg_target_triplet then
					cmake_extra_arguments = cmake_extra_arguments.. [[ -DVCPKG_TARGET_TRIPLET="]]..vcpkg_target_triplet..[["]];
					vcpkg_install_root = make_path(cmake_build_path, "vcpkg_installed", vcpkg_target_triplet);
				end;
			else
				-- no vcpkg.json was found, use installed vcpkg libraries
				vcpkg_install_root = vcpkg_target_triplet and make_path(VCPKG_PATH, "../installed", vcpkg_target_triplet);
				if vcpkg_install_root and not utils:is_dir(vcpkg_install_root) then
					vcpkg_install_root = nil;
				end;
			end;
			if vcpkg_install_root then
				local vcpkg_install_include = make_path(vcpkg_install_root, "include");
				local vcpkg_install_bin = nil;
				local vcpkg_install_lib = nil;
				if config:lower() == "debug" then
					vcpkg_install_bin = make_path(vcpkg_install_root, "debug/bin");
					vcpkg_install_lib = make_path(vcpkg_install_root, "debug/lib");
				else
					vcpkg_install_bin = make_path(vcpkg_install_root, "bin");
					vcpkg_install_lib = make_path(vcpkg_install_root, "lib");
				end;
				table.insert(cmake_bin_path, vcpkg_install_bin);
				table.insert(cmake_inc_path, vcpkg_install_include);
				table.insert(cmake_lib_path, vcpkg_install_lib);
			end;
		end;

		local cmake_project_path = self._home:gsub("\\", "/");
		cmake_build_path = cmake_build_path:gsub("\\", "/");
		local wrap_command = toolset.wrap_command or function(cmd) return cmd; end;
		local map_path = toolset.map_path or function(path) return path; end;

		cmake_exe = wrap_bin_path(cmake_exe, toolset);
		local cmake_generator = [[" -G "Ninja"]];
		if toolset and toolset.tools and type(toolset.tools.cmake_ver) == "number" and toolset.tools.cmake_ver <= 3.17 then
			cmake_generator = [[" -G "CodeBlocks - Ninja"]];
		end;
		local cmake_config_cmd = cmake_exe .. [[ -B "]] ..
			map_path(cmake_build_path) ..
			[[" -S "]] .. map_path(cmake_project_path) ..
			cmake_generator ..
			[[ -DCMAKE_EXPORT_COMPILE_COMMANDS=1]]..
			[[ -DCMAKE_BUILD_TYPE="]] .. config..[["]];
		if cmake_cxx then
			cmake_config_cmd = cmake_config_cmd .. [[ -DCMAKE_CXX_COMPILER="]] .. cmake_cxx .. [["]];
		end;
		if cmake_c then
			cmake_config_cmd = cmake_config_cmd .. [[ -DCMAKE_C_COMPILER="]] .. cmake_c .. [["]];
		end;
		if not toolset.remote and cmake_rc and cmake_rc_path then
			cmake_config_cmd = cmake_config_cmd .. [[ -DCMAKE_RC_COMPILER="]]..cmake_rc..[["]];
		end;
		cmake_config_cmd = cmake_config_cmd..cmake_extra_arguments;
		-- [[" -DCMAKE_MAKE_PROGRAM="]]..cmake_make..

		local def_target = menu_bar.get_cmake_default_target() or { name = ""; output = "" };
		local target_info = self:_find_target_info(build_target) or def_target;
		local ninja_exe = wrap_bin_path(toolset.tools["ninja"] or NINJA_PATH, toolset);
		local ninja_build_cmd = ninja_exe .. [[ -C "]] .. map_path(cmake_build_path) .. [["]];
		local ninja_clean_cmd = ninja_exe .. [[ -C "]] .. map_path(cmake_build_path) .. [[" -t clean]];
		local cmake_clean_cmd = cmake_exe .. [[ --build "]] .. map_path(cmake_build_path) .. [[" --target clean]];
		local cmake_rebuild_target_cmd = cmake_exe .. [[ --build "]] .. map_path(cmake_build_path) .. [[" --clean-first --target]];
		local cmake_rebuild_cmd = cmake_exe .. [[ --build "]] .. map_path(cmake_build_path) .. [[" --clean-first]];
		local cmake_build_target_cmd = cmake_exe .. [[ --build "]] .. map_path(cmake_build_path) .. [[" --target]];
		local cmake_build_cmd = cmake_exe .. [[ --build "]] .. map_path(cmake_build_path) .. [["]];
		local cmake_install_cmd = cmake_exe .. [[ --install "]] .. map_path(cmake_build_path) .. [["]];

		local target_cwd, target_argument, target_environment = self:_find_target_config(toolset, target_info.name, toolset_name, config, cmake_bin_path);
		local debug_type = toolset.tools["debugger"] or "windbg";
		return {
			["type"] = "cmake";
			["config"] = wrap_command(cmake_config_cmd);
			["toolset"] = toolset;
			["install"] = wrap_command(cmake_install_cmd);
			["build"] = wrap_command(cmake_build_cmd);
			["build_target"] = wrap_command(cmake_build_target_cmd .. " " .. make_target_name(target_info));
			["clean"] = wrap_command(cmake_clean_cmd);
			["clean_target"] = wrap_command(ninja_clean_cmd .. " " .. make_target_name(target_info));
			["rebuild"] = wrap_command(cmake_rebuild_cmd);
			-- WARNING!!! CMake rebuild will clean all objects, not only those that belonged to the target
			-- ["rebuild_target"] = wrap_command(cmake_rebuild_target_cmd .. " " .. target_info.name);
			["project_path"] = self._home;
			["build_path"] = cmake_build_path;
			["target_output"] = target_info.output;
			["debug_type"] = debug_type;
			["cwd"] = target_cwd;
			["argument"] = target_argument;
			["environment"] = target_environment;
			["cmake_inc_path"] = cmake_inc_path;
			["cmake_lib_path"] = cmake_lib_path;
			["cmake_bin_path"] = cmake_bin_path;
		};
	end;
	_cmake_gen_do_build_params = function(self, cmd, build_type, on_complete, prepend_cmds, apppend_cmds)
		local cbp_file = utils:list_dir(make_path(cmd.build_path, "*.cbp"));
		local file_api_query_root = make_path(cmd["build_path"], ".cmake/api/v1/query");
		local cmd_lines = {};
		local project_not_configed = self._project_changed or (utils:file_time(make_path(file_api_query_root,"codemodel-v2")) == nil) and (cbp_file == nil);
		if project_not_configed or cmd.config ~= self._last_config[cmd.build_path] then
			-- not configed or config changed
			self._last_config[cmd.build_path] = cmd.config;
			cmd_lines = {
				{
					cmd = cmd.config,
					encoding = "utf-8",
					post_run = function(self, ret_code)
						project._project_changed = false;
					end;
				}
			};
		end;
		local compiler_encoding = "utf-8";
		-- if cmd.toolset["type"] == "msvc" then
		-- 	compiler_encoding = "ansi";
		-- elseif cmd.toolset["type"] == "mingw" and tonumber(string.match(cmd.toolset["version"], "(%d+)")) <= 12 then
		-- 	compiler_encoding = "ansi";
		-- end;
		if build_type then
			if build_type == "rebuild" then
				if cmd["rebuild"] then
					table.insert(cmd_lines,
						{
							cmd = cmd["rebuild"],
							encoding = compiler_encoding
						}
					);
				else
					table.insert(cmd_lines,
						{
							cmd = cmd["clean"],
							encoding = "utf-8"
						}
					);
					table.insert(cmd_lines,
						{
							cmd = cmd["build"],
							encoding = compiler_encoding
						}
					);
				end;
			elseif build_type == "flash" then
				-- esp-idf flash command
				table.insert(cmd_lines,
					{
						cmd = cmd["build_target"],
						encoding = compiler_encoding
					}
				);
			elseif build_type == "install" then
				if cmd["install"] then
					table.insert(cmd_lines, {
							cmd = cmd["build"],
							encoding = compiler_encoding
						}
					);
					table.insert(cmd_lines,
						{
							cmd = cmd["install"],
							encoding = "utf-8"
						}
					);
				else
					table.insert(cmd_lines,
						{
							cmd = cmd["build_target"],
							encoding = compiler_encoding
						}
					);
				end;
			elseif build_type == "rebuild_target" then
				if cmd["rebuild_target"] then
					table.insert(cmd_lines,
						{
							cmd = cmd["rebuild_target"],
							encoding = compiler_encoding
						}
					);
				else
					table.insert(cmd_lines,
						{
							cmd = cmd["clean_target"],
							encoding = "utf-8"
						}
					);
					table.insert(cmd_lines,
						{
							cmd = cmd["build_target"],
							encoding = compiler_encoding
						}
					);
				end;
			elseif build_type == "build_target" or build_type == "build" then
				table.insert(cmd_lines,
					{
						cmd = cmd[build_type],
						encoding = compiler_encoding
					}
				);
			elseif build_type == "build_and_run" then
				table.insert(cmd_lines,
					{
						cmd = cmd["build_target"],
						encoding = compiler_encoding
					}
				);
				local debug_adapter = cmd.toolset and cmd.toolset.debug_adapter;
				local download_cmd = debug_adapter and debug_adapter:make_download_command(
					menu_bar.get_cmake_debug_adapter(),
					menu_bar.get_cmake_debug_adapter_transport(),
					menu_bar.get_cmake_debug_adapter_speed(),
					cmd.target_output,
					cmd.toolset
				);
				if download_cmd then
					table.insert(cmd_lines, download_cmd);
				end;
			else
				table.insert(cmd_lines, cmd[build_type]);
			end;
		else
			cmd_lines = {
				{
					cmd = cmd.config,
					encoding = "utf-8",
				}
			};
			if utils:file_time(make_path(file_api_query_root,"codemodel-v2")) == nil then
				-- cmake project not initialized, should do config twice!
				-- walk-around cmake's issue that the file api does not work when the project was not configed
				table.insert(cmd_lines, {
						cmd = cmd.config,
						encoding = "utf-8"
				});
			end;
		end;

		local toolset = cmd.toolset;
		local begin_clock = os.clock();

		local result_handler_ctx = {["warning"]=0, ["error"]=0};
		local result_handler = self:_find_result_handler(toolset);
		local encoding = toolset.encoding or "utf-8";
		local remote = toolset.remote;

		local unmap_path = function(path) return path; end;
		local map_path = function(path) return path; end;
		if toolset and toolset.unmap_path then
			unmap_path = toolset.unmap_path;
			map_path = toolset.map_path;
		end;

		local param_path = nil;
		if remote then
			param_path = nil;
		else
			local path = table.iclone(toolset.bin);
			if not (toolset.remote or toolset.wsl) then
				local cmake_path = (toolset.tools and toolset.tools["cmake"]) or CMAKE_PATHS[1];
				local gdb_path = (toolset.tools and toolset.tools["gdb"]) or GDB_PATHS[1];
				local ninja_path = (toolset.tools and toolset.tools["ninja"]) or NINJA_PATHS[1];
				local ccache_path = (toolset.tools and toolset.tools["ccache"]) or CCACHE_PATHS[1];
				if cmake_path then
					table.insert(path, make_path(cmake_path, ".."));
				end;
				if gdb_path then
					table.insert(path, make_path(gdb_path, ".."));
				end;
				if ninja_path then
					table.insert(path, make_path(ninja_path, ".."));
				end;
				if ccache_path then
					table.insert(path, make_path(ccache_path, ".."));
				end;
				path = table.iclone(path, cmd.cmake_bin_path);
			end;
			if toolset["target"] == "PICO" then
				local vs_toolset = nil;
				local mgw_toolset = nil;
				for idx, local_toolset in ipairs(TOOLSETS) do
					if local_toolset.name:match("^vs%d%d%d%d%-.*") then
						if vs_toolset == nil then
							vs_toolset = local_toolset;
						end;
					elseif local_toolset.name:match("^gcc%-mgw%d%d%-.*") then
						if mgw_toolset == nil then
							mgw_toolset = local_toolset;
						else
							local local_ver = {local_toolset.version:match("(%d+)%.(%d+)%.(%d+)")};
							local old_ver = {mgw_toolset.version:match("(%d+)%.(%d+)%.(%d+)")};
							if (local_ver[1] > old_ver[1]) or
								(
									local_ver[1] == old_ver[1] and
									(
										local_ver[2] > old_ver[2] or
										(local_ver[2] == old_ver[2] and local_ver[2] > old_ver[2])
									)
								)
							then
								mgw_toolset = local_toolset;
							end;
						end;
					end;
				end;
				if vs_toolset then
					path = table.iclone(path, vs_toolset.bin);
				elseif mgw_toolset then
					path = table.iclone(path, mgw_toolset.bin, mgw_toolset.lib);
				end;
			end;
			param_path = table.concat(table.unique(path), ";")..";"..os.getenv("PATH");
		end;

		local toolset_envs = (toolset.envs or {});
		local toolset_extra_envs = {
			["PATH"] = param_path;
			["CCACHE_DIR"] = toolset_envs["CCACHE_DIR"] or map_path(self:make_path(".edx/ccache"));
			["_EDX_DATA_ROOT"] = edx_appdata_path;
			["_EDX_HOME"] = edx_home_path;
		};

		-- add additional include/library path
		self:_fix_toolset_extra_env(toolset_extra_envs, toolset, cmd);

		if toolset["target"] == "PICO" then
			local vs_toolset = nil;
			local mgw_toolset = nil;
			for idx, local_toolset in ipairs(TOOLSETS) do
				if local_toolset.name:match("^vs%d%d%d%d%-.*") then
					if vs_toolset == nil then
						-- vs_toolset = local_toolset;
					end;
				elseif local_toolset.name:match("^gcc%-mgw%d%d%-.*") then
					if mgw_toolset == nil or cmp_ver(local_toolset.version, mgw_toolset.version) > 0 then
						mgw_toolset = local_toolset;
					end;
				end;
			end;
			if vs_toolset then
				toolset_extra_envs["PATH"] = toolset_extra_envs["PATH"]..table.concat(table.iclone(vs_toolset.bin), ";");
				toolset_extra_envs["INCLUDE"] = table.concat(table.iclone(vs_toolset.include, cmd.cmake_inc_path), ";");
				toolset_extra_envs["LIB"] = table.concat(table.iclone(vs_toolset.lib, cmd.cmake_lib_path), ";");
			elseif mgw_toolset then
				toolset_extra_envs["PATH"] = toolset_extra_envs["PATH"]..table.concat(table.iclone(mgw_toolset.bin, mgw_toolset.lib), ";");
			end;
		end;
		
		local do_build_params = {
			desc = "build";
			cwd = cmd.project_path;
			["encoding"] = encoding;
			["envs"] = table.clone(
				toolset_envs,
				toolset_extra_envs
			);
			["cmds"] = cmd_lines;
			on_output = function(cmd_idx, output_lines)
				if remote and (cmd_idx == 1 or cmd_idx == #cmd_lines) then
					print(output_lines);
					return;
				end;
				dbg_call(
					function()
						result_handler(edx, output_lines, result_handler_ctx, unmap_path);
				end, "");
			end;
			on_command_begin = function(cmd_idx)
				if cmd_idx == 1 then
					edx:clear("build_result");
					menu_bar:disable_cmake_menu_items();
				end;
				local cmd = cmd_lines[cmd_idx];
				if type(cmd) == "table" then
					if cmd["pre_run"] then
						pinvoke(cmd["pre_run"]);
					end;
					print(cmd_idx, " > ", cmd.cmd);
				else
					print(cmd_idx, " > ", cmd);
				end;
			end;
			on_command_end = function(cmd_idx, ret_code)
				print(">>>>");
				local cmd = cmd_lines[cmd_idx];
				if type(cmd) == "table" then
					if cmd["post_run"] then
						pinvoke(cmd["post_run"], self, ret_code);
					end;
				end;
			end;
			on_finished = function(success)
				menu_bar:enable_cmake_menu_items();
				local doc = mgr.current_document;
				doc:reset_render();
				local success_info = "failed";
				if success then
					pinvoke(self._fix_compile_commands_json, self, cmd);
					pinvoke(self._cmake_reload_project_info, self, cmd);
					success_info = "finished";
				end;
				print(("task %s %.3f s, %d error(s), %d warning(s)"):format( success_info, os.clock() - begin_clock, result_handler_ctx["error"], result_handler_ctx["warning"]));
				if on_complete then
					pinvoke(on_complete, success);
				end;
			end;
			["remote"] = remote;
		};

		do
			local prepare_file_api_query =  function(self, ret_code)
				local query_root_file = utils:list_dir(file_api_query_root.."/*");
				if query_root_file == nil then
					utils:mkdir(file_api_query_root);
				end;
				local codemodel_file = make_path(file_api_query_root,"codemodel-v2");
				if utils:file_time(codemodel_file) == nil then
					write_file(codemodel_file, "");
				end;
			end;
			if remote then
				local local_path = file_api_query_root;
				local remote_path = map_path(local_path);
				do_build_params.cwd = remote_path;
				table.insert(cmd_lines, 1, {
						["cmd"] = "@UPLOAD \""..local_path.."\" \""..remote_path.."\"";
						["pre_run"] = prepare_file_api_query;
				});
			else
				local first_cmd = cmd_lines[1];
				if type(first_cmd) ~= "table" then
					cmd_lines[1] = {
						["cmd"] = first_cmd;
					};
					first_cmd = cmd_lines[1];
				end;
				first_cmd["pre_run"] = prepare_file_api_query;
			end;
		end;

		if prepend_cmds then
			for i, val in ipairs(prepend_cmds) do
				table.insert(cmd_lines, i, val);
			end;
		end;
		if apppend_cmds then
			for i, val in ipairs(apppend_cmds) do
				table.insert(cmd_lines, val);
			end;
		end;

		if remote then
			local upload_filter = "";
			if project_configs ~= nil then
				local toolset_name = string.gsub(toolset.name, "[:/\\,*?<>'\"]+", "_");
				local user_config = (project_configs[".targets"] and project_configs[".targets"][".ssh_upload_filter"]) or {};
				upload_filter = user_config[toolset_name];
				if upload_filter then
					upload_filter = " -filter "..upload_filter;
				else
					upload_filter = "";
				end;
			end;
			-- add sync operations
			local local_path = self._home;
			local remote_path = map_path(local_path);
			table.insert(cmd_lines, 1, "@UPLOAD \""..local_path.."\" \""..remote_path.."\""..upload_filter);
			table.insert(cmd_lines, "@DOWNLOAD \""..remote_path.."/.edx/cmake\" \""..local_path.."\\.edx\\cmake\"");

			if cmd.is_install then
				-- download installed files
				cmd_lines[#cmd_lines] = {
					["cmd"] = cmd_lines[#cmd_lines];
					["post_run"] = function()
						local install_manifest = io.u8_open(make_path(cmd.build_path, "install_manifest.txt"));
						if install_manifest == nil then
							print("install_manifest.txt not found!");
							return;
						end;
						local sync_cmds = {};
						for line in install_manifest:lines() do
							local local_path = unmap_path(line, self:make_path("@install-root"));
							table.insert(sync_cmds, "@DOWNLOAD \""..line.."\" \""..local_path.."\"");
						end;
						if #sync_cmds > 0 then
							edx:do_build_append_command(sync_cmds);
							table.append(cmd_lines, table.unpack(sync_cmds));
						end;
						if install_manifest then
							install_manifest:close();
						end;
					end;
				};
			end;
		end;
		return do_build_params;
	end;
	_current_doc_find_build_command = function(self, build_target)
		local explain_vars = function(vars)
			return function(key)
				if vars[key] ~= nil then
					return vars[key];
				end
				return os.getenv(key);
			end;
		end;
		local cur_doc = mgr.current_document;
		if cur_doc.content_type ~= content_type_cxx and
			cur_doc.content_type ~= content_type_dlang
		then
			return nil;
		end;

		local current_file = cur_doc.file_name;
		local current_path = make_path(current_file, "..").."\\";
		local current_file_suffix = current_file:gsub("^.*%.(%w+)$", "%1");
		local is_cpp = current_file_suffix and current_file_suffix:lower() ~= "c";
		local make_compiler_path = function(info)
			local repl_count = 0;
			local cmd, repl = string.gsub(info, ";", function(v)
					repl_count = repl_count + 1;
					if repl_count == 1 then
						return "\" ";
					end;
					return " ";
			end);

			if repl == 0 then
				return [["]] .. info ..[["]];
			end;
			return [["]] .. cmd;
		end;
		local get_compiler = function(tools)
			if is_cpp then
				return make_compiler_path(tools.cxx);
			end;
			return make_compiler_path(tools.cc);
		end;
		local result = { ["type"] = "direct" };
		local build_toolset;
		local build_command;
		local dbg_type, dbg_target;
		local build_lib;
		local build_lib_path;
		local build_include;
		local build_flags;
		local linker_flags = "";
		local dbg_argument = nil;
		local dbg_cwd = nil;
		local dbg_env = {};

		local map_path = function(path) return path; end;
		local wrap_command = function(cmd) return cmd; end;
		local suffixes = {
			exec = ".exe";
			shared = ".dll";
			lib = ".lib";
		};

		for i = 0, 50, 1 do
			local line = mgr.current_document:get_text(i, 0, 2048);
			if line ~= nil then
				local opt_type, opt_value = line:match([[^//%s*([_%w][_%w%d]-)%s*:%s*(.*)]]);
				if opt_type == nil then
					break;
				end;

				if not build_toolset and opt_type == "toolset" then
					build_toolset = opt_value;
				end;

				if not build_command and opt_type == "build" then
					build_command = opt_value;
				end;

				if not build_lib and opt_type == "lib" then
					build_lib = opt_value;
				end;

				if not build_lib_path and opt_type == "lib_path" then
					build_lib_path = opt_value;
				end;

				if not build_include and opt_type == "include" then
					build_include = opt_value;
				end;

				if not build_flags and opt_type == "flags" then
					build_flags = opt_value;
				end;

				if not linker_flags and opt_type == "ld_flags" then
					linker_flags = opt_value;
				end;

				if not dbg_type and opt_type == "debug" then
					dbg_type, dbg_target = opt_value:match([[([^:]-):(.*)]]);
				end;

				if not dbg_argument and opt_type == "args"  then
					dbg_argument = opt_value;
				end;

				if not dbg_cwd and opt_type == "cwd" then
					dbg_cwd = opt_value;
				end;

				if opt_type == "env" then
					local env_key, env_val = opt_value:match([[^([_%w][_%w%d]-)%s*=%s*(.*)$]]);
					if env_key then
						dbg_env[env_key] = env_val;
					end;
				end;
			end;
		end;

		if TOOLSETS and (build_toolset or not self:is_ready()) then
			for i, ts in ipairs(TOOLSETS) do
				if ts.name == build_toolset then
					result.toolset = ts;
					break;
				end;
			end;
			if not result.toolset and build_toolset then
				result.toolset = TOOLSETS[tonumber(build_toolset)];
			end;
			if not result.toolset then
				result.toolset = TOOLSETS[1];
			end;
			if result.toolset then
				local toolset = result.toolset;
				map_path = toolset.map_path or map_path;
				wrap_command = toolset.wrap_command or wrap_command;
				suffixes = toolset.suffixes or suffixes;
				if dbg_env["PATH"] then
					dbg_env["PATH"] = table.concat(table.unique({dbg_env["PATH"], table.unpack(toolset.bin), table.unpack(toolset.lib)}), ";");
				else
					dbg_env["PATH"] = table.concat(table.unique({table.unpack(toolset.bin), table.unpack(toolset.lib)}), ";");
				end;
			end;
		end;

		local explainer = explain_vars({
				["THIS_FILE"] = map_path(current_file);
				["THIS_FILE:PATH"] = map_path(current_path);
				["THIS_FILE:BASENAME"] = get_base_name(current_file);
				["THIS_FILE:NAME"] = get_file_name(current_file);
		});

		if build_command == nil and result.toolset then
			if result.toolset.tools.msvc then
				-- use msvc cl command line
				build_command = [[]] .. get_compiler(result.toolset.tools) .. [[ /Zi ${THIS_FILE} /Fe:${THIS_FILE:PATH}${THIS_FILE:BASENAME}]] .. suffixes["exec"]
				dbg_type = result.toolset.tools["debugger"] or [[windbg]]
			else
				-- use gcc command line
				build_command = [[]] .. get_compiler(result.toolset.tools) .. [[ -g ${THIS_FILE} -o ${THIS_FILE:PATH}${THIS_FILE:BASENAME}]] .. suffixes["exec"];
				dbg_type = result.toolset.tools["debugger"] or [[gdb/mi]];
			end;
			if dbg_target == nil or dbg_target == "" then
				dbg_target = [[${THIS_FILE:PATH}${THIS_FILE:BASENAME}]] .. suffixes["exec"];
			end;
		end;

		if dbg_type ~= nil then
			dbg_target = string.gsub(dbg_target, [[%${([-.%$%w%d:_]+)}]], explainer);
			result.debug_type = dbg_type;
			result.target_output = dbg_target;
		end;

		if dbg_cwd ~= nil then
			result.cwd = dbg_cwd;
		end;

		if dbg_argument ~= nil then
			result.argument = dbg_argument;
		end;

		if build_command ~= nil then
			if build_flags then
				if result.toolset then
					build_flags = " "..build_flags;
					if result.toolset.tools.msvc then
						build_flags = build_flags:gsub("%s%-Wa,%-mbig%-obj", " /bigobj");
						build_flags = build_flags:gsub("%s%-fexceptions", " /EHsc");
						build_flags = build_flags:gsub("%s%-fno%-exceptions", " /EHsc-");
						build_flags = build_flags:gsub("%s%-O3", " /Ox");
						build_flags = build_flags:gsub("%s%-O0", " /Od");
						build_flags = build_flags:gsub("%s%-mtune=%S*", "");
						build_flags = build_flags:gsub("%s%-fstack%-check=no", " /GS-");
						build_flags = build_flags:gsub("%s%-fstack%-check=%S+", " /GS");
						build_flags = build_flags:gsub("%s%-fno%-stack%-check", " /GS-");
						build_flags = build_flags:gsub("%s%-fstack%-check", " /GS");
						build_flags = build_flags:gsub("%s%-std=gnu%+%+", " /std:c++");
						build_flags = build_flags:gsub("%s%-std=", " /std:");
						if build_flags:match("%s%-static%-libstdc%+%+") then
							build_flags = build_flags:gsub("%s%-static%-libstdc%+%+", " /MT");
							build_flags = build_flags:gsub("%s%-static%-libgcc", "");
							build_flags = build_flags:gsub("%s%-static", "");
							build_flags = build_flags:gsub("%s%-dn", "");
							build_flags = build_flags:gsub("%s%-non_shared", "");
						elseif build_flags:match("%s%-static%-libgcc") then
							build_flags = build_flags:gsub("%s%-static%-libgcc", " /MT");
							build_flags = build_flags:gsub("%s%-static", "");
							build_flags = build_flags:gsub("%s%-dn", "");
							build_flags = build_flags:gsub("%s%-non_shared", "");
						else
							build_flags = build_flags:gsub("%s%-static", " /MT");
							build_flags = build_flags:gsub("%s%-non_shared", " /MT");
							build_flags = build_flags:gsub("%s%-dn", " /MT");
						end;
						build_flags = build_flags:gsub("%s%-Bdynamic", " /MD");
						build_flags = build_flags:gsub("%s%-dy", " /MD");
						build_flags = build_flags:gsub("%s%-call_shared", " /MD");
						build_flags = build_flags:gsub("%s%-fPIC", "");
						build_flags = build_flags:gsub("%s%-fPIE", "");
						build_flags = build_flags:gsub("%s%-fpic", "");
						build_flags = build_flags:gsub("%s%-fpie", "");
						build_flags = build_flags:gsub("%s%-g", " /Zi");
						build_flags = build_flags:gsub("%s%-ggdb", " /Zi");
						build_flags = build_flags:gsub("%s%-gcoff", " /Zi");
						build_flags = build_flags:gsub("%s%-gdwarf", " /Zi");
						if build_flags:match("%s%-s") or build_flags:match("%s%-S") then
							build_flags = build_flags:gsub("%s%-s", " ");
							build_flags = build_flags:gsub("%s%-S", " ");
							linker_flags = linker_flags .. " /Release";
						end;
						if build_flags:match("%s/[Rr]elease") then
							build_flags = build_flags:gsub("%s/[Rr]elease", " ");
							linker_flags = linker_flags .. " /Release";
						end;
						if build_flags:match("%s/[Dd]ebug") then
							build_flags = build_flags:gsub("%s/[Dd]ebug", " ");
							linker_flags = linker_flags .. " /Debug";
						end;
						if build_flags:match("%s/mwindows") then
							build_flags = build_flags:gsub("%s/mwindows", " ");
							linker_flags = linker_flags .. " /SUBSYSTEM:Windows";
						end;
						build_flags = build_flags:gsub("%s%-flto", " /GL");

						build_flags = build_flags:gsub("%s%-", " /");
						if result.toolset.version == 17 then
							build_flags = build_flags:gsub("%s/std:c%+%+2[3ab]", " /std:c++latest");
							build_flags = build_flags:gsub("%s/std:c%+%+20", " /std:c++20");
						else
							build_flags = build_flags:gsub("%s/std:c%+%+2[03ab]", " /std:c++latest");
						end
						build_flags = build_flags:gsub("%s/std:c%+%+1z", " /std:c++17");
						build_flags = build_flags:gsub("%s/std:c%+%+1[1y]", " /std:c++14");
						build_flags = build_flags:gsub("%s/std:c%+%+0x", " /std:c++14");
						build_flags = build_flags:gsub("%s/std:c%+%+98", " /std:c++14");

						if build_flags:match("%s/Zi%-") then
							build_flags = build_flags:gsub("%s%/Zi%-", " ");
							build_flags = build_flags:gsub("%s%/Zi", " ");
							build_command = build_command:gsub("%s%/Zi", " ");
						elseif build_flags:match("%s/Z7") then
							build_flags = build_flags:gsub("%s%/Zi", " ");
							build_command = build_command:gsub("%s%/Zi", " ");
						end;
					else
						build_flags = build_flags:gsub("%s/EHsc%-", " -fno-exceptions");
						build_flags = build_flags:gsub("%s/GX%-", " -fno-exceptions");
						build_flags = build_flags:gsub("%s/EHsc", " -fexceptions");
						build_flags = build_flags:gsub("%s/GX", " -fexceptions");
						build_flags = build_flags:gsub("%s/Ox", " -O3 -mtune=native");
						build_flags = build_flags:gsub("%s/Od", " -O0");
						build_flags = build_flags:gsub("%s/GS%-", " -fstack-check=no");
						build_flags = build_flags:gsub("%s/GS", " -fstack-check");
						build_flags = build_flags:gsub("%s/Ge", " -fstack-check");
						build_flags = build_flags:gsub("%s/std:", " -std=");
						build_flags = build_flags:gsub("%s/bigobj", " -Wa,-mbig-obj");
						build_flags = build_flags:gsub("%s/MTd", " -static -static-libgcc -static-libstdc++");
						build_flags = build_flags:gsub("%s/MT", " -static -static-libgcc -static-libstdc++");
						build_flags = build_flags:gsub("%s/MD", " ");
						build_flags = build_flags:gsub("%s/MDd", " ");
						build_flags = build_flags:gsub("%s/GL%-", " ");
						build_flags = build_flags:gsub("%s/GL", " -flto");
						if build_flags:match("%s/Zi%-") then
							build_flags = build_flags:gsub("%s/Zi%-", " -s");
						end;
						if build_flags:match("%s/Z7") then
							build_flags = build_flags:gsub("%s%/Z7", " ");
						end;

						build_flags = build_flags:gsub("%s/Zi", " -g");
						build_flags = build_flags:gsub("%s/Debug", " -g");
						build_flags = build_flags:gsub("%s/Release", " -s");

						build_flags = build_flags:gsub("%s%-std=c%+%+latest", " -std=c++20");

						build_flags = build_flags:gsub("%s/", " -");

						if build_flags:match("%s%-s%s") or build_flags:match("%s%-s$") then
							build_command = build_command:gsub("%s%-g", " ");
						end;
					end;
					build_flags = build_flags:gsub("^%s*", "");
				end;
				build_command = build_command .. " " .. build_flags;
			end

			if build_include and result.toolset then
				if result.toolset.tools.msvc then
					build_include = build_include:gsub("([-_%d%w%+%./\\]+)([,;$]?)", " /I%1");
				else
					build_include = build_include:gsub("([-_%d%w%+%./\\]+)([,;$]?)", " -I%1");
				end;
				build_command = build_command .. build_include;
			end;

			if build_lib and result.toolset then
				local lib_search_paths = nil;
				if build_lib_path then
					lib_search_paths = {build_lib_path:match("([-_%d%w%+%./\\]+)[,;$]?")};
				end;
				local is_lib_file_exists = function()
					return false;
				end;
				if lib_search_paths then
					is_lib_file_exists = function(name)
						for i, path in ipairs(lib_search_paths) do
							lib_path = make_path(current_path, path, name);
							if utils:file_time(lib_path) then
								return true;
							end;
						end;
						return false;
					end;
				end;

				if result.toolset.tools.msvc then
					build_lib = build_lib:gsub("([-_%d%w%+%.]+)([,;$]?)", function(name)
							if is_lib_file_exists(name) then
								return " "..name;
							end;
							return " "..name..".lib";
					end);
				else
					build_lib = build_lib:gsub("([-_%d%w%+%.]+)([,;$]?)", function(name)
							if is_lib_file_exists(name) then
								local sname = name:match("^lib(.*)%.a$");
								if sname then
									return " -l"..sname;
								end;
								return " "..name;
							end;
							return " -l"..name;
					end);
				end;
				build_command = build_command .. build_lib;
			end;

			if build_lib_path and result.toolset then
				if result.toolset.tools.msvc then
					build_lib_path = build_lib_path:gsub("([-_%d%w%+%./\\]+)([,;$]?)", " /LIBPATH:%1");
					build_command = build_command .. " /link" .. build_lib_path;
				else
					build_lib_path = build_lib_path:gsub("([-_%d%w%+%./\\]+)([,;$]?)", " -L%1");
					build_command = build_command .. build_lib_path;
				end;
			end;

			if linker_flags and result.toolset and result.toolset["type"] == "msvc" then
				if linker_flags:match("%s/[Rr]elease") then
					linker_flags = linker_flags:gsub("%s/[Dd]ebug", " ");
					build_command = build_command:gsub("%s/Zi%s", " ");
				end;
				if build_command:match("%s/Zi") then
					linker_flags = linker_flags:gsub("%s/[Dd]ebug", " ");
				end;
				linker_flags = linker_flags:gsub("%s+", " ");
				if linker_flags ~= nil and linker_flags ~= " " and linker_flags ~= "" then
					if build_command:match("%s/link%s") then
						build_command = build_command .. linker_flags;
					else
						build_command = build_command .. " /link" .. linker_flags;
					end;
				end;
			end;

			local cmd = string.gsub(build_command, [[%${([-.%$%w%d:_]+)}]], explainer);

			local dbg_target_time = dbg_target and utils:file_time(dbg_target);
			if dbg_target_time and dbg_target_time >= utils:file_time(current_file) then
				result.build = "cmd /d /c echo nothing to do!";
			else
				result.build = wrap_command(cmd);
			end;
			result.project_path = current_path;
		end;

		if not result.cwd then
			result.cwd = current_path;
		end;

		if dbg_env then
			local env_cache = {};
			for item_key, val in pairs(dbg_env) do
				val = string.gsub(val, [[%${([-.%$%w%d:_]+)}]],
					function(key)
						local env_val = env_cache[key];
						if env_val then
							return env_val;
						end;
						env_val = explainer(key);
						env_cache[key] = env_val;
						return env_val;
					end
				);
				env_cache[item_key] = val;
				dbg_env[item_key] = val;
			end;
			result.environment = dbg_env;
		end;
		if result.build or result.target_output then
			return result;
		end;
	end;
	_current_doc_gen_do_build_params = function(self, cmd, build_type, on_complete, prepend_cmds, apppend_cmds)
		local cmd_lines = {};
		if build_type then
			if build_type == "build_target" or build_type == "build_and_run" then
				build_type = "build"
			end;
			if build_type == "rebuild" then
				table.insert(cmd_lines, cmd["clean"]);
				table.insert(cmd_lines, cmd["build"]);
			else
				table.insert(cmd_lines, cmd[build_type]);
			end;
		else
			cmd_lines = {cmd.config};
		end;

		if prepend_cmds then
			for i, val in ipairs(prepend_cmds) do
				table.insert(cmd_lines, i, val);
			end;
		end;
		if apppend_cmds then
			for i, val in ipairs(apppend_cmds) do
				table.insert(cmd_lines, val);
			end;
		end;

		local toolset = cmd.toolset or {bin={};include={};lib={};tools={}};
		local tools = toolset.tools;
		local param_path = {table.unpack(toolset.bin), table.unpack(toolset.lib)};
		table.insert(param_path, os.getenv("PATH"));

		local begin_clock = os.clock();

		local result_handler_ctx = {["warning"]=0,["error"]=0};

		local result_handler = handle_msvc_build_result;

		local encoding = toolset.encoding or "";

		local remote = toolset.remote;

		if tools.clang then
			if tools.msvc then
				result_handler = handle_clang_cl_build_result;
			else
				result_handler = handle_clang_build_result;
			end;
		elseif tools.gcc then
			result_handler = handle_gcc_build_result;
		elseif tools.msvc then
			result_handler = handle_msvc_build_result;
		end;

		local unmap_path = function(path) return path; end;
		if toolset and toolset.unmap_path then
			unmap_path = toolset.unmap_path;
		end;

		local do_build_params = {
			desc = "build";
			cwd = cmd.project_path;
			["encoding"] = encoding;
			["envs"] = {
				["PATH"] = table.concat(param_path,  ";");
				["INCLUDE"] = table.concat(toolset.include, ";");
				["LIB"] = table.concat(toolset.lib, ";");
			};
			["cmds"] = cmd_lines;
			on_output = function(cmd_idx, output_lines)
				result_handler(edx, output_lines, result_handler_ctx, unmap_path);
			end;
			on_command_begin = function(cmd_idx)
				if cmd_idx == 1 then
					edx:clear("build_result");
				end;
				print(cmd_idx, " > ", cmd_lines[cmd_idx]);
			end;
			on_command_end = function(cmd_idx, ret_code)
				print(">>>>");
			end;
			on_finished = function(success)
				local doc = mgr.current_document;
				doc:reset_render();
				local success_info = "failed";
				if success then
					success_info = "finished";
				end;
				print(("task %s %.3f s, %d error(s), %d warning(s)"):format( success_info, os.clock() - begin_clock, result_handler_ctx["error"], result_handler_ctx["warning"]));
				if on_complete then
					on_complete(success);
				end;
			end;
			["remote"] = remote;
		};

		return do_build_params;
	end;
	_xmake_load = function(self, dir, xmake_file)
		self._home = dir;
		self._type = "xmake";
		self._last_config = {};
		self._xmake_file = xmake_file;
		menu_bar:show_cmake_menu();
		menu_bar:show_debug_menu();
		self:reload_settings();

		local build = project_configs and project_configs[".build"];
		if build and build.config then
			menu_bar.set_cmake_config(build.config);
		end;
		if build and build.toolset then
			menu_bar.set_cmake_toolset(build.toolset);
		end;
		local toolset = menu_bar.get_cmake_toolset(true);
		if toolset and toolset.tools and toolset.tools["xmake"] and not toolset.remote and not toolset.wsl then
			self._xmake_bin = toolset.tools["xmake"];
		else
			self._xmake_bin = XMAKE_PATHS[1];
		end;
		if not toolset.remote and not toolset.wsl and (self._xmake_bin == nil or utils:file_time(self._xmake_bin) == nil) then
			print("\x1b[33;41m", LANG("XMake not found!"), "\x1b[0m");
			print("\x1b[33;41m", LANG("Download from https://xmake.io"), "\x1b[0m");
			return false;
		end;

		if toolset then
			self:on_toolset_changed(toolset);
		end;
		local save_project_config = false;

		if project_configs == nil then
			project_configs = empty_project_configs();
			save_project_config = true;
		end;

		-- load cmake targets
		local cmd = self:_xmake_find_build_command(nil);
		if cmd ~= nil and cmd.type == "xmake" then
			local do_build_params = self:_xmake_gen_do_build_params(cmd, nil,function()
					local build = project_configs[".build"];
					if build and build.target then
						menu_bar.set_cmake_default_target(build.target);
						xws_mgr:update_targets({menu_bar.get_cmake_default_target()});
					end;
					lsp:reset(cmd);
					cmake_update_include_path_for_opened_documents();

					if save_project_config then
						self:save_settings();
					end;
					-- refresh document structure
					lsp:list_doc_structure(mgr.current_document, nil);
			end);
			self._last_config[cmd.build_path] = cmd.config;
			toolchain_update_include_path_for_opened_documents(); -- use toolchain include path by default
			edx.do_build(do_build_params);
			__last_toolset = cmd.toolset;
			local editor = project_configs and project_configs[".editor"];
			if editor and editor.opened then
				local paths = {};
				for i,filename in ipairs(editor.opened) do
					table.insert(paths, self:make_path(filename));
				end;
				mgr.open_doc(paths);
			end;
		end;
		menu_bar.set_cmake_text(LANG("menu_bar/cmake/xmake"));
		return true;
	end;
	_xmake_unload = function(self)
		if project_configs == nil then
			project_configs = empty_project_configs();
		end;
		-- close all opened project files
		local all_doc = {mgr.get_all_documents()};
		local all_doc_name = {};
		local all_doc_states = project_configs[".editor"].states or {};
		for idx, doc in ipairs(all_doc) do
			local name = doc.file_name;
			local rel_path = self:make_rel_path(name);
			if rel_path ~= nil then
				table.insert(all_doc_name, rel_path);
				all_doc_states[rel_path] = {
					cursor = {doc.cursor_line;doc.cursor_column;};
					fold = doc.fold_status;
				};
			end;
		end;
		mgr.close_doc(all_doc);
		project_configs[".editor"] = {
			opened = all_doc_name;
			states = all_doc_states;
		};
		menu_bar.set_cmake_text(LANG("menu_bar/cmake"));
		menu_bar.show_cmake_items();
		menu_bar.hide_cmake_items("serial_ports", "serial_baud", "debug_adapters", "cpus", "boards");
		self:save_settings();
		self._xmake_file = nil;
		self._xmake_bin = nil;
		self._xmake_targets = nil;
		self._type = nil;
		self._home = nil;
		project_configs = nil;
	end;
	_xmake_find_build_command = function(self, build_target)
		local toolset = menu_bar.get_cmake_toolset(true);
		local map_path = toolset.map_path or function(path) return path; end;
		local unmap_path = toolset.unmap_path or function(path) return path; end;
		local config = menu_bar.get_cmake_config();
		local toolset_name = string.gsub(toolset.name, "[:/\\,*?<>'\"]+", "_");
		local xmake_build_path = self:make_path(".edx/xmake/" .. toolset_name .. "/" .. config);
		local xmake_project_home = map_path(self._home);

		if not toolset.remote and not toolset.wsl and (self._xmake_bin == nil or utils:file_time(self._xmake_bin) == nil) then
			return nil
		end;

		local xmake_bin = self._xmake_bin;
		local toolset = menu_bar.get_cmake_toolset(true);
		local xmake_extra_arguments = "";
		if project_configs and project_configs[".xmake"] then
			local xmake_config_info = project_configs[".xmake"];
			if xmake_config_info.path then
				if utils:file_time(xmake_config_info.path) == nil then
					-- print("user specified cmake [".. xmake_config_info.path.."] is invalid!");
				else
					xmake_bin = xmake_config_info.path;
				end;
			end;
			local scope = config.." - "..toolset.name;
			if xmake_config_info.arguments and xmake_config_info.arguments[scope] then
				xmake_extra_arguments = xmake_extra_arguments.." " .. table.concat(xmake_config_info.arguments[scope], " ");
			end;
			if type(xmake_config_info.argument) == "string" then
				xmake_extra_arguments = xmake_extra_arguments.." " .. project_configs[".xmake"].argument;
			elseif type(xmake_config_info.argument) == "table" then
				xmake_extra_arguments = xmake_extra_arguments.." " .. table.concat(project_configs[".xmake"].argument, " ");
			end;
		end;

		if toolset["remote"] or toolset["target"] == "wsl"then
			xmake_bin = "xmake";
		elseif toolset.tools and toolset.tools["xmake"] then
			xmake_bin = toolset.tools["xmake"];
		end;
		local xmake_debug_flag = "";
		local xmake_rebuild_cache_flag = "";
		if (edx:get_key_status("shift") and not edx:get_key_status("alt") and not edx:get_key_status("ctrl")) then
			xmake_debug_flag = "-vD";
		end;
		if build_target == ".rebuild_cache" then
			xmake_rebuild_cache_flag = " -c"
			build_target = nil;
		end;
		local xmake_bin_path = {make_path(xmake_bin, "..")};
		xmake_bin = wrap_bin_path(xmake_bin, toolset);
		local xmake_config_cmd = xmake_bin .. [[ f ]]..xmake_debug_flag..xmake_rebuild_cache_flag..xmake_extra_arguments..[[ -y -P "]]..xmake_project_home..[[" --buildir="]] .. map_path(xmake_build_path) .. "\" -p ";
		local tools = toolset.tools;
		local toolset_arch = toolset.target:match("(.-)%-.*");
		local cxx_compiler, cxx_flags = split_string(tools.cxx, ";");
		local c_compiler, c_flags = split_string(tools.cc or tools.cxx, ";");
		local ld_flags = nil;
		if toolset["type"] == "msvc" then
			xmake_config_cmd = xmake_config_cmd..[[windows --toolchain=msvc]];
			if toolset_arch == "x86_64" then
				toolset_arch = "x64";
			end;
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. c_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. cxx_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --includedirs="]] .. table.concat(toolset.include, ";") .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --linkdirs="]] .. table.concat(toolset.lib, ";") .. [["]];
			-- local vs_ver = toolset.name:match("vs(%d+)%-.*");
			-- if vs_ver then
			-- 	xmake_config_cmd = xmake_config_cmd..[[ --vs=]]..vs_ver;
			-- end;
			-- xmake_config_cmd = xmake_config_cmd..[[ --vs_toolset=]]..toolset.version;
			-- xmake_config_cmd = xmake_config_cmd..[[ --vs_sdkver=]]..toolset.sdk;
		elseif toolset["type"] == "mingw" then
			if toolset["name"]:match("^gcc%-msys%-.*") then
				xmake_config_cmd = xmake_config_cmd..[[msys --toolchain=mingw --sdk="]]..toolset.home..[["]];
			else
				xmake_config_cmd = xmake_config_cmd..[[mingw --toolchain=mingw --mingw="]]..toolset.home..[["]];
			end;

			if not c_compiler:match("^(.*[%-%./\\])gcc[%-%./\\].*") then
				c_compiler = "gcc@"..c_compiler;
			end;

			if not cxx_compiler:match("^(.*[%-%./\\])g%+%+[%-%./\\].*") then
				cxx_compiler = "g++@"..cxx_compiler;
			end;

			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. c_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. cxx_compiler .. [["]];
			if toolset["name"]:match(".*%-m32") then
				xmake_config_cmd = xmake_config_cmd.. [[ --ld="]] .. cxx_compiler .. [["]];
				ld_flags = cxx_flags;
			end;
		elseif toolset["type"] == "llvm-mingw" then
			if not c_compiler:match("^(.*[%-%./\\])clang[%-%./\\].*") then
				c_compiler = "clang@"..c_compiler;
			end;

			if not cxx_compiler:match("^(.*[%-%./\\])clang%+%+[%-%./\\].*") then
				cxx_compiler = "clang++@"..cxx_compiler;
			end;

			xmake_config_cmd = xmake_config_cmd..[[mingw --toolchain=llvm --sdk="]]..toolset.home..[["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. c_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. cxx_compiler .. [["]];
		elseif toolset["type"] == "clang-msvc" then
			if not c_compiler:match("^(.*[%-%./\\])clang[%-%./\\].*") then
				c_compiler = "clang@"..c_compiler;
			end;

			if not cxx_compiler:match("^(.*[%-%./\\])clang%+%+[%-%./\\].*") then
				cxx_compiler = "clang++@"..cxx_compiler;
			end;

			xmake_config_cmd = xmake_config_cmd..[[windows]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. c_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. cxx_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --includedirs="]] .. table.concat(toolset.include, ";") .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --linkdirs="]] .. table.concat(toolset.lib, ";") .. [["]];
		elseif toolset["type"] == "clang-cl" then
			xmake_config_cmd = xmake_config_cmd..[[windows]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="clang-cl@]] .. c_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="clang-cl@]] .. cxx_compiler .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --includedirs="]] .. table.concat(toolset.include, ";") .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --linkdirs="]] .. table.concat(toolset.lib, ";") .. [["]];
			c_flags = nil;
			cxx_flags = nil;
		elseif toolset.wsl then
			xmake_config_cmd = xmake_config_cmd..[[linux]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. map_path(c_compiler) .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. map_path(cxx_compiler) .. [["]];
			toolset_arch = "x86_64";
		elseif toolset.remote then
			xmake_config_cmd = xmake_config_cmd..[[linux]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cc="]] .. map_path(c_compiler) .. [["]];
			xmake_config_cmd = xmake_config_cmd.. [[ --cxx="]] .. map_path(cxx_compiler) .. [["]];
		else
			print("\x1b[33;41m", "Unsupported toolset! ", toolset["name"], "/", toolset["type"], "\x1b[0m");
		end;
		if c_flags then
			xmake_config_cmd = xmake_config_cmd.. [[ --cflags="]] .. c_flags .. [["]];
		end;
		if cxx_flags then
			xmake_config_cmd = xmake_config_cmd.. [[ --cxxflags="]] .. cxx_flags .. [["]];
		end;
		if ld_flags then
			xmake_config_cmd = xmake_config_cmd.. [[ --ldflags="]] .. ld_flags .. [["]];
		end;
		if tools.rc then
			local rc_compiler, rc_flags = split_string(tools.rc, ";");
			if rc_compiler then
				xmake_config_cmd = xmake_config_cmd.. [[ --mrc="]] .. rc_compiler .. [["]];
				if rc_flags then
					xmake_config_cmd = xmake_config_cmd.. [[ --mrcflags="]] .. rc_flags .. [["]];
				end;
			end;
		end;

		xmake_config_cmd = xmake_config_cmd.. [[ --arch=]] .. toolset_arch;
		xmake_config_cmd = xmake_config_cmd.. [[ --mode=]] .. config:lower();
		
		local xmake_build_cmd = xmake_bin .. [[ build ]]..xmake_debug_flag..[[ -y -P "]]..xmake_project_home..[["]];
		local xmake_rebuild_cmd = xmake_bin .. [[ build ]]..xmake_debug_flag..[[ -y -r -P "]]..xmake_project_home..[["]];
		local xmake_clean_cmd = xmake_bin .. [[ clean ]]..xmake_debug_flag..[[ -P "]]..xmake_project_home..[["]];
		local xmake_install_cmd = xmake_bin .. [[ install ]]..xmake_debug_flag..[[ -P "]]..xmake_project_home..[["]];
		local output_path = map_path(xmake_build_path);
		if toolset.remote then
			output_path = output_path:sub(#xmake_project_home + 2);
		end;
		local xmake_compile_commands_cmd = xmake_bin .. [[ project -y -P "]]..xmake_project_home..[[" -k compile_commands --lsp=clangd --archs="]]..toolset_arch..[[" --modes="]] .. config:lower()..[[" "]]..output_path..[["]];
		local xmake_compile_ninja_cmd = xmake_bin .. [[ project -y -P "]]..xmake_project_home..[[" -k ninja --archs="]]..toolset_arch..[[" --modes="]] .. config:lower()..[[" "]]..output_path..[["]];

		local wrap_command = toolset.wrap_command or function(cmd) return cmd; end;
		local def_target = menu_bar.get_cmake_default_target() or { name = ""; output = "" };
		local target_info = self:_find_target_info(build_target) or def_target;
		local target_cwd, target_argument, target_environment = self:_find_target_config(toolset, target_info.name, toolset_name, config, xmake_bin_path);
		local debug_type = toolset.tools["debugger"] or "windbg";
		return {
			["type"] = "xmake";
			["config"] = wrap_command(xmake_config_cmd);
			["toolset"] = toolset;
			["install"] = wrap_command(xmake_install_cmd);
			["build"] = wrap_command(xmake_build_cmd) .. " -a";
			["compile_commands"] = wrap_command(xmake_compile_commands_cmd);
			["compile_ninja"] = wrap_command(xmake_compile_ninja_cmd);
			["build_target"] = wrap_command(xmake_build_cmd .. " " .. make_target_name(target_info));
			["rebuild"] = wrap_command(xmake_rebuild_cmd);
			["rebuild_target"] = wrap_command(xmake_rebuild_cmd .. " " .. make_target_name(target_info));
			["clean"] = wrap_command(xmake_clean_cmd);
			["clean_target"] = wrap_command(xmake_clean_cmd .. " " .. make_target_name(target_info));
			["project_path"] = self._home;
			["build_path"] = xmake_build_path;
			["target_output"] = target_info.output;
			["debug_type"] = debug_type;
			["cwd"] = target_cwd;
			["argument"] = target_argument;
			["environment"] = target_environment;
			["cmake_inc_path"] = cmake_inc_path;
			["cmake_lib_path"] = cmake_lib_path;
			["cmake_bin_path"] = cmake_bin_path;
		};
	end;
	_xmake_gen_do_build_params = function(self, cmd, build_type, on_complete, prepend_cmds, apppend_cmds)
		local xmake_config_path_exists = utils:is_dir(make_path(self._home, ".xmake"));
		local cmd_lines = {};
		if not xmake_config_path_exists or cmd.config ~= self._last_config[cmd.build_path] then
			-- not configed or config changed
			self._last_config[cmd.build_path] = cmd.config;
			cmd_lines = {
				{cmd = cmd.config, encoding = "utf-8"};
				{cmd = cmd.compile_commands, encoding = "utf-8"};
				{cmd = cmd.compile_ninja, encoding = "utf-8"};
			};
		end;

		if build_type and build_type ~= ".rebuild_cache" and build_type ~= "build_and_run" then
			table.insert(cmd_lines, cmd[build_type]);
		else
			cmd_lines = {
				{cmd = cmd.config, encoding = "utf-8"};
				{cmd = cmd.compile_commands, encoding = "utf-8"};
				{cmd = cmd.compile_ninja, encoding = "utf-8"};
			};
		end;

		if prepend_cmds then
			for i, val in ipairs(prepend_cmds) do
				table.insert(cmd_lines, i, val);
			end;
		end;
		if apppend_cmds then
			for i, val in ipairs(apppend_cmds) do
				table.insert(cmd_lines, val);
			end;
		end;

		local toolset = cmd.toolset;
		local begin_clock = os.clock();

		local result_handler_ctx = {["warning"]=0, ["error"]=0};
		local result_handler = self:_find_result_handler(toolset);
		local encoding = toolset.encoding or "";
		local remote = toolset.remote;

		local unmap_path = function(path) return path; end;
		local map_path = function(path) return path; end;
		if toolset and toolset.unmap_path then
			unmap_path = toolset.unmap_path;
			map_path = toolset.map_path;
		end;

		local param_path = nil;
		if remote then
			param_path = nil;
		else
			local path = table.iclone(toolset.bin);
			if not (toolset.remote or toolset.wsl) then
				local cmake_path = (toolset.tools and toolset.tools["cmake"]) or CMAKE_PATHS[1];
				local gdb_path = (toolset.tools and toolset.tools["gdb"]) or GDB_PATHS[1];
				local ninja_path = (toolset.tools and toolset.tools["ninja"]) or NINJA_PATHS[1];
				local ccache_path = (toolset.tools and toolset.tools["ccache"]) or CCACHE_PATHS[1];
				if cmake_path then
					table.insert(path, make_path(cmake_path, ".."));
				end;
				if gdb_path then
					table.insert(path, make_path(gdb_path, ".."));
				end;
				if ninja_path then
					table.insert(path, make_path(ninja_path, ".."));
				end;
				if ccache_path then
					table.insert(path, make_path(ccache_path, ".."));
				end;
				path = table.iclone(path, cmd.cmake_bin_path);
			end;
			param_path = table.concat(table.unique(path), ";")..";"..os.getenv("PATH");
		end;

		local toolset_envs = (toolset.envs or {});
		local toolset_extra_envs = {
			["PATH"] = param_path;
			["CCACHE_DIR"] = toolset_envs["CCACHE_DIR"] or map_path(self:make_path(".edx/ccache"));
		};

		-- add additional include/library path
		self:_fix_toolset_extra_env(toolset_extra_envs, toolset, cmd);

		local do_build_params = {
			desc = "build";
			cwd = cmd.project_path;
			["encoding"] = encoding;
			["envs"] = table.clone(
				toolset_envs,
				toolset_extra_envs
			);
			["cmds"] = cmd_lines;
			on_output = function(cmd_idx, output_lines)
				if remote and (cmd_idx == 1 or cmd_idx == #cmd_lines) then
					print(output_lines);
					return;
				end;
				dbg_call(
					function()
						result_handler(edx, output_lines, result_handler_ctx, unmap_path);
				end, "");
			end;
			on_command_begin = function(cmd_idx)
				if cmd_idx == 1 then
					edx:clear("build_result");
					menu_bar:disable_cmake_menu_items();
				end;
				if type(cmd_lines[cmd_idx]) == "table" then
					print(cmd_idx, " > ", cmd_lines[cmd_idx].cmd);
				else
					print(cmd_idx, " > ", cmd_lines[cmd_idx]);
				end;
			end;
			on_command_end = function(cmd_idx, ret_code)
				print(">>>>");
				local cmd = cmd_lines[cmd_idx];
				if type(cmd) == "table" then
					if cmd["post_run"] then
						pinvoke(cmd["post_run"], self, ret_code);
					end;
				end;
			end;
			on_finished = function(success)
				menu_bar:enable_cmake_menu_items();
				local doc = mgr.current_document;
				doc:reset_render();

				local success_info = "failed";
				if success then
					pinvoke(function()
							self:_fix_compile_commands_json(cmd);
					end);
					pinvoke(function()
							self:_xmake_refresh_targets(cmd);
					end);
					success_info = "finished";
				end;
				print(("task %s %.3f s, %d error(s), %d warning(s)"):format( success_info, os.clock() - begin_clock, result_handler_ctx["error"], result_handler_ctx["warning"]));
				if on_complete then
					pinvoke(on_complete, success);
				end;
			end;
			["remote"] = remote;
		};

		if remote then
			local upload_filter = "";
			if project_configs ~= nil then
				local toolset_name = string.gsub(toolset.name, "[:/\\,*?<>'\"]+", "_");
				local user_config = (project_configs[".targets"] and project_configs[".targets"][".ssh_upload_filter"]) or {};
				upload_filter = user_config[toolset_name];
				if upload_filter then
					upload_filter = " -filter "..upload_filter;
				else
					upload_filter = "";
				end;
			end;
			-- add sync operations
			local local_path = self._home;
			local remote_path = map_path(local_path);
			do_build_params.cwd = remote_path;
			table.insert(cmd_lines, 1, "@UPLOAD \""..local_path.."\" \""..remote_path.."\""..upload_filter);
			table.insert(cmd_lines, {
					["cmd"] = "@DOWNLOAD \""..remote_path.."/.edx/xmake\" \""..local_path.."\\.edx\\xmake\"";
					["post_run"] = function(self, ret_code)
						if ret_code ~= 0 then
							return;
						end;
						-- download xmake libraries
						local ninja_build_file = make_path(cmd.build_path, "build.ninja");
						local targets = self:_xmake_extract_target_info(ninja_build_file, cmd.build_path);

						local xmake_include_paths = {};
						for name, target in pairs(targets) do
							if target.include then
								for idx, path in ipairs(target.include) do
									if path:match("^[\\/].*[\\/]%.xmake[\\/]packages[\\/].*$") then
										table.insert(xmake_include_paths, path);
									end;
								end;
							end;
						end;
						xmake_include_paths = table.unique(xmake_include_paths);
						local sync_cmds = {};
						for idx, path in ipairs(xmake_include_paths) do
							local local_path = unmap_path(path);
							table.insert(sync_cmds, "@DOWNLOAD \""..path.."\" \""..local_path.."\"");
						end;
						if #sync_cmds > 0 then
							edx:do_build_append_command(sync_cmds);
							table.append(cmd_lines, table.unpack(sync_cmds));
						end;
					end;
			});
		end;
		return do_build_params;
	end;
	_xmake_extract_target_info = function(self, ninja_build_file, base_path)
		local file = io.u8_open(ninja_build_file, "r");
		if file == nil then
			return {};
		end;
		local target_map = {}
		local last_target = nil;

		local format_path = function(path)
			if path:match("^[/].*$") then
				return path;
			end;
			if path:match("^%w:[\\/].*$") then
				return abs_path(path);
			end;
			return make_path(base_path, path);
		end;

		local extract_include = function(args)
			local include_head = nil;
			local in_string = false;
			local idx = 1;
			local includes = {};
			args = args:gsub([["%-I\"]], [[-I"]]);
			args = args:gsub([["%-isystem\"]], [[-isystem"]]);
			args = args:gsub([[\""]], [["]]);
			while idx <= #args do
				if include_head then
					local ch = args:sub(idx, idx);
					if include_head == idx and ch == "\"" then
						in_string = true;
						idx = idx + 1;
					elseif ch == "\\" then
						idx = idx + 2;
					elseif in_string then
						if ch == "\"" then
							in_string = false;
							-- end of include
							table.insert(includes, format_path(args:sub(include_head+1, idx-1)))
							idx = idx + 1;
							include_head = nil;
						else
							idx = idx + 1;
						end;
					elseif ch == " " then
						-- end of include
						table.insert(includes, format_path(args:sub(include_head, idx-1)))
						idx = idx + 1;
						include_head = nil;
					else
						idx = idx + 1;
					end;
				elseif args:sub(idx, idx+1) == "-I" then
					include_head = idx + 2;
					idx = idx + 2;
				elseif args:sub(idx, idx+10) == "-external:I" then
					include_head = idx + 11;
					idx = idx + 11;
				elseif args:sub(idx, idx+7) == "-isystem" then
					include_head = idx + 8;
					idx = idx + 8;
					if args:sub(idx, idx) == " " then
						include_head = include_head + 1;
						idx = idx + 1;
					end;
				else
					idx = idx + 1;
				end;
			end;
			if include_head then
				table.insert(includes, format_path(args:sub(include_head)))
			end;
			return includes;
		end;

		for line in file:lines() do
			if #line > 6 then
				local target = line:match([[^# build target: (.+)]]);
				if target then
					target_map[target] = {
						["name"] = target;
						["include"] = {};
					};
					goto __NEXT;
				end;

				local target, file = line:match([[^build (.*): phony (.+)]]);
				if target then
					target = target:gsub([["(.*)"]], "%1");
					file = file:gsub([["(.*)"]], "%1");
					if target_map[target] then
						last_target = target_map[target];
						last_target["output"] = format_path(file);
						if file:match([[.*%.dll]]) or file:match([[.*%.so]]) then
							last_target["type"] = 3;
						elseif file:match([[.*%.a]]) or file:match([[.*%.lib]]) then
							last_target["type"] = 2;
						else
							last_target["type"] = 1;
						end;
					end;
					goto __NEXT;
				end;

				local target_args = line:match([[^ ARGS = (.*)]]);
				if target_args then
					last_target.include = table.iclone(last_target.include, extract_include(target_args));
				end;
			end;

			::__NEXT::
		end;
		file:close();
		return target_map;
	end;
	_xmake_refresh_targets = function(self, cmd)
		local ninja_build_file = make_path(cmd.build_path, "build.ninja");
		local targets = self:_xmake_extract_target_info(ninja_build_file, cmd.build_path);
		local unmap_path = cmd.toolset.unmap_path or function(path) return path; end;
		local map_path = cmd.toolset.map_path or function(path) return path; end;
		self._xmake_targets = {};
		for name, target in pairs(targets) do
			target.include = table.iclone(target.include, cmd.toolset.include);
			target.include = table.unique(target.include);
			for idx, path in ipairs(target.include) do
				target.include[idx] = unmap_path(path);
			end;
			target.output = map_path(target.output);
			if cmd.toolset.remote then
				if target.output and target.output:sub(1,1) == "/" then
					target.output = target.output;
				else
					target.output = cmd.toolset.home.."/"..target.output;
				end;
			elseif target["type"] == 1 and utils:file_time(target.output) == nil and utils:file_time(target.output..".exe") then
				target.output = target.output..".exe";
			end;
			table.insert(self._xmake_targets, target);
		end;
		xws_mgr:update_targets(self._xmake_targets, true);
		menu_bar.update_cmake_target(self._xmake_targets);
	end;
	regular_path = function(self, path)
		if not self:is_ready() or not self:is_xmake() then
			return edx:regular_path(path);
		end;
		path = path:gsub("\x1b%[.-m", "");
		local real_path = path:match("^error:%s+(.*)$");
		if real_path == nil then
			real_path = path:match("^warning:%s+(.*)$");
		end;
		if real_path then
			path = real_path;
		end;
		if not path:match("^%w:[\\/].*$") then
			path = self:make_path(path);
		end;
		return edx:regular_path(path);
	end;

};
